@@ -736,8 +736,8 @@ gnome_canvas_bpath_update_gdk (GnomeCanvasBpath * bpath, double * affine, ArtSVP
 
 	/* Free subpath lists */
 
-	while (gdk->closed_paths) gdk->closed_paths = g_slist_remove_link (gdk->closed_paths, gdk->closed_paths);
-	while (gdk->open_paths) gdk->open_paths = g_slist_remove_link (gdk->open_paths, gdk->open_paths);
+	while (gdk->closed_paths) gdk->closed_paths = g_slist_remove (gdk->closed_paths, gdk->closed_paths->data);
+	while (gdk->open_paths) gdk->open_paths = g_slist_remove (gdk->open_paths, gdk->open_paths->data);
 
 	/* Calcualte new GdkPoints array and subpath lists */
 
@@ -798,7 +798,7 @@ gnome_canvas_bpath_update_gdk (GnomeCanvasBpath * bpath, double * affine, ArtSVP
 			}
 
 			gp_path_unref (path);
-			clist = g_slist_remove_link (clist, clist);
+			clist = g_slist_remove (clist, clist->data);
 		}
 
 		/* Fill GdkPoints and add subpaths to list: open subpaths */
@@ -827,7 +827,7 @@ gnome_canvas_bpath_update_gdk (GnomeCanvasBpath * bpath, double * affine, ArtSVP
 			}
 
 			gp_path_unref (path);
-			olist = g_slist_remove_link (olist, olist);
+			olist = g_slist_remove (olist, olist->data);
 		}
 
 	}
@@ -911,7 +911,7 @@ gnome_canvas_bpath_update (GnomeCanvasItem *item, double *affine, ArtSVP *clip_p
 	if ((priv->outline_set) && (!gp_path_is_empty (priv->path))) {
 		gdouble width;
 		ArtBpath * abp;
-		ArtVpath * vpath;
+		ArtVpath * vpath, * pvpath;
 
 		/* Set linewidth */
 
@@ -930,25 +930,28 @@ gnome_canvas_bpath_update (GnomeCanvasItem *item, double *affine, ArtSVP *clip_p
 		vpath = art_bez_path_to_vec (abp, 0.25);
 		art_free (abp);
 
+		pvpath = art_vpath_perturb (vpath);
+		art_free (vpath);
+
 		/* If dashed, apply dash */
 
 		if (priv->dash.dash != NULL)
 		{
-			ArtVpath *old = vpath;
+			ArtVpath *old = pvpath;
 			
-			vpath = art_vpath_dash (old, &priv->dash);
+			pvpath = art_vpath_dash (old, &priv->dash);
 			art_free (old);
 		}
 		
 		/* Stroke vpath to SVP */
 
-		svp = art_svp_vpath_stroke (vpath,
+		svp = art_svp_vpath_stroke (pvpath,
 					    gnome_canvas_join_gdk_to_art (priv->join),
 					    gnome_canvas_cap_gdk_to_art (priv->cap),
 					    width,
 					    priv->miterlimit,
 					    0.25);
-		art_free (vpath);
+		art_free (pvpath);
 
 		if (item->canvas->aa) {
 
@@ -1127,9 +1130,9 @@ gcbp_destroy_gdk (GnomeCanvasBpath * bpath)
 			g_free (gdk->points);
 
 		while (gdk->closed_paths)
-			gdk->closed_paths = g_slist_remove_link (gdk->closed_paths, gdk->closed_paths);
+			gdk->closed_paths = g_slist_remove (gdk->closed_paths, gdk->closed_paths->data);
 		while (gdk->open_paths)
-			gdk->open_paths = g_slist_remove_link (gdk->open_paths, gdk->open_paths);
+			gdk->open_paths = g_slist_remove (gdk->open_paths, gdk->open_paths->data);
 
 		if (gdk->ctx)
 			gcbp_draw_ctx_unref (gdk->ctx);
