@@ -447,9 +447,10 @@ gnome_font_face_get_sample (const GnomeFontFace * face)
 			*p = '\0';
 			return s;
 		} else {
+			/* Nope - we do not know coverage for now */
 			p = s;
-			for (i = 0; i < 32; i++ ) {
-				p += g_unichar_to_utf8 (0xe000 + i, p);
+			for (i = 1; (i < 33) && (i < face->private->num_glyphs); i++ ) {
+				p += g_unichar_to_utf8 (face->private->glyphs[i].unicode, p);
 			}
 			*p = '\0';
 			return s;
@@ -1062,6 +1063,7 @@ gff_load_afm (GnomeFontFace * face)
 				info = fi->cmi + i;
 
 				unicode = gp_unicode_from_ps (info->name);
+				if (unicode < 1) unicode = gp_unicode_from_dingbats (info->name);
 
 				if (unicode < 1) {
 					if (!priv->privencoding) priv->privencoding = g_hash_table_new (g_str_hash, g_str_equal);
@@ -1123,6 +1125,7 @@ gff_load_afm (GnomeFontFace * face)
 		      GnomeFontLigList *ll;
 		      gint u;
 
+		      /* We assume here that dingbats do not have ligatures */
 		      u = gp_unicode_from_ps (ligs->succ);
 		      succ = GPOINTER_TO_INT (g_hash_table_lookup (face->private->glyphmap, GINT_TO_POINTER (u)));
 		      u = gp_unicode_from_ps (ligs->succ);
@@ -1157,6 +1160,7 @@ gff_load_afm (GnomeFontFace * face)
 	      int j;
 	      gint u;
 
+	      /* We assume here that dingbats do not have kerning */
 	      u = gp_unicode_from_ps (fi->pkd[i].name1);
 	      glyph1 = GPOINTER_TO_INT (g_hash_table_lookup (face->private->glyphmap, GINT_TO_POINTER (u)));
 	      u = gp_unicode_from_ps (fi->pkd[i].name2);
@@ -1258,7 +1262,7 @@ gnome_font_face_get_xheight (const GnomeFontFace * face)
 }
 
 const ArtDRect *
-gnome_font_face_get_font_bbox (const GnomeFontFace * face)
+gnome_font_face_get_stdbbox (const GnomeFontFace * face)
 {
 	g_return_val_if_fail (face != NULL, NULL);
 	g_return_val_if_fail (GNOME_IS_FONT_FACE (face), NULL);
