@@ -20,6 +20,10 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* Must include these two first */
+#include "config.h"
+#include <libgnomeprint/gnome-print-i18n.h>
+
 #include <gnome.h>
 #include <libgnome/gnome-defs.h>
 #include <libgnomeprint/gnome-printer.h>
@@ -28,6 +32,23 @@
 
 #include "gnome-print-dialog.h"
 
+struct _GnomePrintDialog {
+	GnomeDialog dialog;
+
+	void *priv;
+
+	GtkWidget *printer;
+	GtkWidget *copies;
+	GnomePrintRangeType range_type;
+	GtkWidget *range;
+	GtkWidget *range_container;
+	GtkAccelGroup *range_accel_group;
+};
+
+struct _GnomePrintDialogClass {
+	GnomeDialogClass parent_class;
+};
+
 static void gnome_print_dialog_class_init (GnomePrintDialogClass *class);
 static void gnome_print_dialog_init       (GnomePrintDialog      *gspaper);
 
@@ -130,17 +151,13 @@ static void init_body(GnomePrintDialog *gpd, int flags)
 		gtk_box_pack_start(GTK_BOX(GNOME_DIALOG (gpd)->vbox), GTK_WIDGET (hbox), FALSE, FALSE, 3);
 
 		if (flags & GNOME_PRINT_DIALOG_COPIES) {
-			GtkSpinButton *sb;
 			GnomePrintCopies *gpc;
 			
 			gpd->copies = gnome_print_copies_new ();
 			gtk_box_pack_end(GTK_BOX(hbox), gpd->copies, FALSE, TRUE, 3);
 			gpc = GNOME_PRINT_COPIES (gpd->copies);
-			sb = GTK_SPIN_BUTTON (gpc->copies);
-			gnome_dialog_editable_enters
-				(GNOME_DIALOG(gpd), GTK_EDITABLE(&sb->entry));
-			gtk_window_add_accel_group (GTK_WINDOW (gpd),
-						    gpc->accel_group);
+			gnome_print_copies_bind_editable_enters (gpc, GNOME_DIALOG (gpd));
+			gnome_print_copies_bind_accel_group (gpc, GTK_WINDOW (gpd));
 		}
 		
 		if (flags & GNOME_PRINT_DIALOG_RANGE) {
@@ -153,16 +170,24 @@ static void init_body(GnomePrintDialog *gpd, int flags)
 		}
 		gtk_widget_show_all(hbox);
 	}
-	gnome_dialog_editable_enters (GNOME_DIALOG (gpd),
-				      GTK_EDITABLE (gpw->entry_command));
-	gnome_dialog_editable_enters (GNOME_DIALOG (gpd),
-				      GTK_EDITABLE (gpw->entry_filename));
-	
-	gtk_window_add_accel_group (GTK_WINDOW (gpd), gpw->accel_group);
+
+	gnome_printer_widget_bind_editable_enters (gpw, GNOME_DIALOG (gpd));
+	gnome_printer_widget_bind_accel_group (gpw, GTK_WINDOW (gpd));
+#if 0
 	gtk_widget_grab_focus (gpw->profile_selector);
+#endif
 }
 
-static const char *print_buttons[] = { _("Print"), _("Preview"), GNOME_STOCK_BUTTON_CANCEL, NULL };
+static char const * *
+print_buttons()
+{
+	static char const *values[4];
+	values [0] = _("Print");
+	values [1] = _("Preview");
+	values [2] = GNOME_STOCK_BUTTON_CANCEL;
+	values [3] = NULL;
+	return values;
+}
 
 /**
  * gnome_print_dialog_new:
@@ -185,7 +210,7 @@ gnome_print_dialog_new (const char *title, int flags)
 	GtkWidget *w;
 
 	w = GTK_WIDGET ( gtk_type_new (gnome_print_dialog_get_type ()));
-	gnome_dialog_constructv(GNOME_DIALOG(w), title, print_buttons);
+	gnome_dialog_constructv(GNOME_DIALOG(w), title, print_buttons());
 	init_body(GNOME_PRINT_DIALOG(w), flags);
 	return w;
 }
@@ -201,7 +226,7 @@ gnome_print_dialog_new (const char *title, int flags)
 void
 gnome_print_dialog_construct (GtkWidget *gpd, const char *title, int flags)
 {
-	gnome_dialog_constructv(GNOME_DIALOG(gpd), title, print_buttons);
+	gnome_dialog_constructv(GNOME_DIALOG(gpd), title, print_buttons());
 	init_body(GNOME_PRINT_DIALOG(gpd), flags);
 }
 
@@ -271,12 +296,14 @@ gnome_print_dialog_construct_range_any(GnomePrintDialog *gpd, int flags, GtkWidg
 	g_return_if_fail(gpd != NULL);
 	g_return_if_fail(GNOME_IS_PRINT_DIALOG(gpd));
 	g_return_if_fail(gpd->range_container != NULL);
-
+	g_return_if_fail(!(flags&GNOME_PRINT_RANGE_SELECTION && flags&GNOME_PRINT_RANGE_SELECTION_UNSENSITIVE));
 	p = gpd->priv;
 
 	row=0;
 	if (flags&GNOME_PRINT_RANGE_SELECTION)
 		row++;
+	if (flags&GNOME_PRINT_RANGE_SELECTION_UNSENSITIVE)
+		row++;
 	if (flags&GNOME_PRINT_RANGE_ALL)
 		row++;
 	if (flags&GNOME_PRINT_RANGE_RANGE)
@@ -343,6 +370,16 @@ gnome_print_dialog_construct_range_any(GnomePrintDialog *gpd, int flags, GtkWidg
 						    GDK_MOD1_MASK, 0);
 		row++;
 	}
+	if (flags&GNOME_PRINT_RANGE_SELECTION_UNSENSITIVE) {
+		selection = gtk_radio_button_new_with_label(group, "");
+		accel_key = gtk_label_parse_uline
+			(GTK_LABEL (GTK_BIN (selection)->child),
+			 _("_Selection"));
+		gtk_widget_set_sensitive (GTK_WIDGET(selection), FALSE);
+		gtk_table_attach(GTK_TABLE(table), selection, 0, 1, row, row+1, GTK_FILL, GTK_FILL, 0, 0);
+		group = gtk_radio_button_group((GtkRadioButton *)selection);
+		row++;
+	}
 
 	replace(&p->current, current);
 	replace(&p->all, all);
@@ -616,15 +653,13 @@ gnome_print_dialog_get_range_date(GnomePrintDialog *gpd, time_t *start, time_t *
 void
 gnome_print_dialog_get_copies(GnomePrintDialog *gpd, int *copies, int *collate)
 {
-	int ncopies=1, ncollate=FALSE;
-
 	if (gpd->copies) {
-		gnome_print_copies_get_copies((GnomePrintCopies *)gpd->copies, &ncopies, &ncollate);
+		if (copies) *copies = gnome_print_copies_get_copies((GnomePrintCopies *) gpd->copies);
+		if (collate) *collate = gnome_print_copies_get_collate ((GnomePrintCopies *) gpd->copies);
+	} else {
+		if (copies) *copies = 1;
+		if (collate) *collate = FALSE;
 	}
-	if (copies)
-		*copies = ncopies;
-	if (collate)
-		*collate = ncollate;
 }
 
 /**
