@@ -52,8 +52,10 @@ int gnome_print_encode_deflate_wcs (gint size);
 int gnome_print_encode_ascii85_ (const guchar *in, guchar *out, gint in_size);
 
 /* gnome-print-encode virtual stopwatch */
+#if 0 /* disabled by chema 20-oct-2000 */
 void gnome_print_encode_timer_start (void);
 void gnome_print_encode_timer_end (void);
+#endif
 
 END_GNOME_DECLS
 
