@@ -42,7 +42,6 @@ struct _GnomePrintFRGBAClass {
 
 struct _GnomePrintFRGBAPrivate {
 	GnomePrintContext * context;
-	GPGC * gc;
 	GnomePrintMeta * meta;
 };
 
@@ -50,10 +49,9 @@ static void gpf_class_init (GnomePrintFRGBAClass *class);
 static void gpf_init (GnomePrintFRGBA *frgba);
 static void gpf_destroy (GtkObject *object);
 
-static gint gpf_fill (GnomePrintContext * pc);
-static gint gpf_eofill (GnomePrintContext * pc);
+static gint gpf_fill (GnomePrintContext * pc, ArtWindRule rule);
 static gint gpf_stroke (GnomePrintContext * pc);
-static gint gpf_show_ucs4 (GnomePrintContext * pc, guint32 * buf, gint length);
+static gint gpf_show_sized (GnomePrintContext * pc, const char *text, int bytes);
 static gint gpf_textline (GnomePrintContext * pc, GnomeTextLine * line);
 
 static gint gpf_grayimage (GnomePrintContext * pc, const gchar * data, gint width, gint height, gint rowstride);
@@ -81,12 +79,8 @@ static gint gpf_setopacity (GnomePrintContext * pc, gdouble opacity);
 static gint gpf_setfont (GnomePrintContext * pc, GnomeFont * font);
 
 static gint gpf_concat (GnomePrintContext * pc, const gdouble matrix[6]);
-#ifdef SETMATRIX_DEPRECATED
-static gint gpf_setmatrix (GnomePrintContext * pc, const gdouble matrix[6]);
-#endif
 
-static gint gpf_clip (GnomePrintContext * pc);
-static gint gpf_eoclip (GnomePrintContext * pc);
+static gint gpf_clip (GnomePrintContext * pc, ArtWindRule rule);
 
 static gint gpf_beginpage (GnomePrintContext * pc, const gchar * name);
 static gint gpf_showpage (GnomePrintContext * pc);
@@ -141,9 +135,8 @@ gpf_class_init (GnomePrintFRGBAClass *class)
 	object_class->destroy = gpf_destroy;
 	
 	pc_class->fill = gpf_fill;
-	pc_class->eofill = gpf_eofill;
 	pc_class->stroke = gpf_stroke;
-	pc_class->show_ucs4 = gpf_show_ucs4;
+	pc_class->show_sized = gpf_show_sized;
 	pc_class->textline = gpf_textline;
 	pc_class->grayimage = gpf_grayimage;
 	pc_class->rgbimage = gpf_rgbimage;
@@ -167,14 +160,10 @@ gpf_class_init (GnomePrintFRGBAClass *class)
 	pc_class->setfont = gpf_setfont;
 
 	pc_class->concat = gpf_concat;
-#ifdef SETMATRIX_DEPRECATED
-	pc_class->setmatrix = gpf_setmatrix;
-#endif		
 	pc_class->gsave = gpf_gsave;
 	pc_class->grestore = gpf_grestore;
 
 	pc_class->clip = gpf_clip;
-	pc_class->eoclip = gpf_eoclip;
 
 	pc_class->beginpage = gpf_beginpage;
 	pc_class->showpage = gpf_showpage;
@@ -188,7 +177,6 @@ gpf_init (GnomePrintFRGBA * frgba)
 	g_assert (frgba->private != NULL);
 
 	frgba->private->context = NULL;
-	frgba->private->gc = NULL;
 	frgba->private->meta = NULL;
 }
 
@@ -202,8 +190,6 @@ gpf_destroy (GtkObject *object)
 	if (frgba->private) {
 		if (frgba->private->context)
 			gtk_object_unref (GTK_OBJECT (frgba->private->context));
-		if (frgba->private->gc)
-			gp_gc_unref (frgba->private->gc);
 		if (frgba->private->meta)
 			gtk_object_unref (GTK_OBJECT (frgba->private->meta));
 		g_free (frgba->private);
@@ -218,7 +204,7 @@ gpf_destroy (GtkObject *object)
  */
 
 static gint
-gpf_fill (GnomePrintContext * pc)
+gpf_fill (GnomePrintContext * pc, ArtWindRule rule)
 {
 	GnomePrintFRGBA * frgba;
 	GnomePrintFRGBAPrivate * fp;
@@ -229,57 +215,30 @@ gpf_fill (GnomePrintContext * pc)
 	frgba = GNOME_PRINT_FRGBA (pc);
 	fp = frgba->private;
 
-	gnome_print_fill (GNOME_PRINT_CONTEXT (fp->meta));
+	if (rule == ART_WIND_RULE_NONZERO) {
+		gnome_print_fill (GNOME_PRINT_CONTEXT (fp->meta));
+	} else {
+		gnome_print_eofill (GNOME_PRINT_CONTEXT (fp->meta));
+	}
 
-	if (gp_gc_get_opacity (fp->gc) <= (255.0 / 256.0)) {
+	if (gp_gc_get_opacity (pc->gc) <= (255.0 / 256.0)) {
 		/* We have alpha! */
 		ArtDRect bbox;
 
-		gpf_currentpath_bbox (fp->gc, &bbox);
+		gpf_currentpath_bbox (pc->gc, &bbox);
 
 		gnome_print_gsave (frgba->private->context);
 		gnome_print_clip (frgba->private->context);
 		gpf_render_buf (frgba, &bbox);
 		gnome_print_grestore (frgba->private->context);
 	} else {
-		return gnome_print_fill (fp->context);
+		if (rule == ART_WIND_RULE_NONZERO) {
+			return gnome_print_fill (fp->context);
+		} else {
+			return gnome_print_fill (fp->context);
+		}
 	}
 
-	gp_gc_newpath (fp->gc);
-
-	return 1;
-}
-
-static gint
-gpf_eofill (GnomePrintContext * pc)
-{
-	GnomePrintFRGBA * frgba;
-	GnomePrintFRGBAPrivate * fp;
-
-	g_return_val_if_fail (pc != NULL, 0);
-	g_return_val_if_fail (GNOME_IS_PRINT_FRGBA (pc), 0);
-
-	frgba = GNOME_PRINT_FRGBA (pc);
-	fp = frgba->private;
-
-	gnome_print_eofill (GNOME_PRINT_CONTEXT (fp->meta));
-
-	if (gp_gc_get_opacity (fp->gc) <= (255.0 / 256.0)) {
-		/* We have alpha! */
-		ArtDRect bbox;
-
-		gpf_currentpath_bbox (fp->gc, &bbox);
-
-		gnome_print_gsave (frgba->private->context);
-		gnome_print_eoclip (frgba->private->context);
-		gpf_render_buf (frgba, &bbox);
-		gnome_print_grestore (frgba->private->context);
-	} else {
-		return gnome_print_eofill (fp->context);
-	}
-
-	gp_gc_newpath (fp->gc);
-
 	return 1;
 }
 
@@ -297,26 +256,21 @@ gpf_stroke (GnomePrintContext * pc)
 	frgba = GNOME_PRINT_FRGBA (pc);
 	fp = frgba->private;
 
-	gp_gc_newpath (fp->gc);
 	gnome_print_stroke (GNOME_PRINT_CONTEXT (fp->meta));
 	return gnome_print_stroke (fp->context);
 }
 
 static gint
-gpf_show_ucs4 (GnomePrintContext * pc, guint32 * buf, gint length)
+gpf_show_sized (GnomePrintContext * pc, const char *text, int bytes)
 {
 	GnomePrintFRGBA * frgba;
 	GnomePrintFRGBAPrivate * fp;
 
-	g_return_val_if_fail (pc != NULL, 0);
-	g_return_val_if_fail (GNOME_IS_PRINT_FRGBA (pc), 0);
-
 	frgba = GNOME_PRINT_FRGBA (pc);
 	fp = frgba->private;
 
-	gp_gc_newpath (fp->gc);
-	gnome_print_show_ucs4 (GNOME_PRINT_CONTEXT (fp->meta), buf, length);
-	return gnome_print_show_ucs4 (fp->context, buf, length);
+	gnome_print_show_sized (GNOME_PRINT_CONTEXT (fp->meta), text, bytes);
+	return gnome_print_show_sized (fp->context, text, bytes);
 }
 
 static gint
@@ -331,7 +285,6 @@ gpf_textline (GnomePrintContext * pc, GnomeTextLine * line)
 	frgba = GNOME_PRINT_FRGBA (pc);
 	fp = frgba->private;
 
-	gp_gc_newpath (fp->gc);
 	gnome_print_textline (GNOME_PRINT_CONTEXT (fp->meta), line);
 	return gnome_print_textline (fp->context, line);
 }
@@ -396,7 +349,7 @@ gpf_rgbaimage (GnomePrintContext * pc, const gchar * data, gint width, gint heig
 
 	gnome_print_rgbaimage (GNOME_PRINT_CONTEXT (fp->meta), data, width, height, rowstride);
 
-	ctm = gp_gc_get_ctm (frgba->private->gc);
+	ctm = gp_gc_get_ctm (pc->gc);
 
 	p.x = 0.0;
 	p.y = 0.0;
@@ -456,7 +409,6 @@ gpf_gsave (GnomePrintContext * pc)
 
 	frgba = GNOME_PRINT_FRGBA (pc);
 
-	gp_gc_gsave (frgba->private->gc);
 	gnome_print_gsave (GNOME_PRINT_CONTEXT (frgba->private->meta));
 	return gnome_print_gsave (frgba->private->context);
 }
@@ -471,7 +423,6 @@ gpf_grestore (GnomePrintContext * pc)
 
 	frgba = GNOME_PRINT_FRGBA (pc);
 
-	gp_gc_grestore (frgba->private->gc);
 	gnome_print_grestore (GNOME_PRINT_CONTEXT (frgba->private->meta));
 	return gnome_print_grestore (frgba->private->context);
 }
@@ -491,7 +442,6 @@ gpf_newpath (GnomePrintContext * pc)
 
 	frgba = GNOME_PRINT_FRGBA (pc);
 
-	gp_gc_newpath (frgba->private->gc);
 	gnome_print_newpath (GNOME_PRINT_CONTEXT (frgba->private->meta));
 	return gnome_print_newpath (frgba->private->context);
 }
@@ -506,7 +456,6 @@ gpf_moveto (GnomePrintContext * pc, gdouble x, gdouble y)
 
 	frgba = GNOME_PRINT_FRGBA (pc);
 
-	gp_gc_moveto (frgba->private->gc, x, y);
 	gnome_print_moveto (GNOME_PRINT_CONTEXT (frgba->private->meta), x, y);
 	return gnome_print_moveto (frgba->private->context, x, y);
 }
@@ -521,7 +470,6 @@ gpf_lineto (GnomePrintContext * pc, gdouble x, gdouble y)
 
 	frgba = GNOME_PRINT_FRGBA (pc);
 
-	gp_gc_lineto (frgba->private->gc, x, y);
 	gnome_print_lineto (GNOME_PRINT_CONTEXT (frgba->private->meta), x, y);
 	return gnome_print_lineto (frgba->private->context, x, y);
 }
@@ -536,7 +484,6 @@ gpf_curveto (GnomePrintContext * pc, gdouble x1, gdouble y1, gdouble x2, gdouble
 
 	frgba = GNOME_PRINT_FRGBA (pc);
 
-	gp_gc_curveto (frgba->private->gc, x1, y1, x2, y2, x3, y3);
 	gnome_print_curveto (GNOME_PRINT_CONTEXT (frgba->private->meta), x1, y1, x2, y2, x3, y3);
 	return gnome_print_curveto (frgba->private->context, x1, y1, x2, y2, x3, y3);
 }
@@ -551,7 +498,6 @@ gpf_closepath (GnomePrintContext * pc)
 
 	frgba = GNOME_PRINT_FRGBA (pc);
 
-	gp_gc_closepath (frgba->private->gc);
 	gnome_print_closepath (GNOME_PRINT_CONTEXT (frgba->private->meta));
 	return gnome_print_closepath (frgba->private->context);
 }
@@ -566,7 +512,6 @@ gpf_strokepath (GnomePrintContext * pc)
 
 	frgba = GNOME_PRINT_FRGBA (pc);
 
-	gp_gc_strokepath (frgba->private->gc);
 	gnome_print_strokepath (GNOME_PRINT_CONTEXT (frgba->private->meta));
 	return gnome_print_strokepath (frgba->private->context);
 }
@@ -585,7 +530,6 @@ gpf_setrgbcolor (GnomePrintContext * pc, gdouble r, gdouble g, gdouble b)
 
 	frgba = GNOME_PRINT_FRGBA (pc);
 
-	gp_gc_set_rgbcolor (frgba->private->gc, r, g, b);
 	gnome_print_setrgbcolor (GNOME_PRINT_CONTEXT (frgba->private->meta), r, g, b);
 	return gnome_print_setrgbcolor (frgba->private->context, r, g, b);
 }
@@ -600,7 +544,6 @@ gpf_setopacity (GnomePrintContext * pc, gdouble opacity)
 
 	frgba = GNOME_PRINT_FRGBA (pc);
 
-	gp_gc_set_opacity (frgba->private->gc, opacity);
 	gnome_print_setopacity (GNOME_PRINT_CONTEXT (frgba->private->meta), opacity);
 	return gnome_print_setopacity (frgba->private->context, opacity);
 }
@@ -615,7 +558,6 @@ gpf_setlinewidth (GnomePrintContext * pc, gdouble width)
 
 	frgba = GNOME_PRINT_FRGBA (pc);
 
-	gp_gc_set_linewidth (frgba->private->gc, width);
 	gnome_print_setlinewidth (GNOME_PRINT_CONTEXT (frgba->private->meta), width);
 	return gnome_print_setlinewidth (frgba->private->context, width);
 }
@@ -630,7 +572,6 @@ gpf_setmiterlimit (GnomePrintContext * pc, gdouble limit)
 
 	frgba = GNOME_PRINT_FRGBA (pc);
 
-	gp_gc_set_miterlimit (frgba->private->gc, limit);
 	gnome_print_setmiterlimit (GNOME_PRINT_CONTEXT (frgba->private->meta), limit);
 	return gnome_print_setmiterlimit (frgba->private->context, limit);
 }
@@ -645,7 +586,6 @@ gpf_setlinejoin (GnomePrintContext * pc, gint jointype)
 
 	frgba = GNOME_PRINT_FRGBA (pc);
 
-	gp_gc_set_linejoin (frgba->private->gc, jointype);
 	gnome_print_setlinejoin (GNOME_PRINT_CONTEXT (frgba->private->meta), jointype);
 	return gnome_print_setlinejoin (frgba->private->context, jointype);
 }
@@ -660,7 +600,6 @@ gpf_setlinecap (GnomePrintContext * pc, gint captype)
 
 	frgba = GNOME_PRINT_FRGBA (pc);
 
-	gp_gc_set_linecap (frgba->private->gc, captype);
 	gnome_print_setlinecap (GNOME_PRINT_CONTEXT (frgba->private->meta), captype);
 	return gnome_print_setlinecap (frgba->private->context, captype);
 }
@@ -675,7 +614,6 @@ gpf_setdash (GnomePrintContext * pc, gint n_values, const gdouble * values, gdou
 
 	frgba = GNOME_PRINT_FRGBA (pc);
 
-	gp_gc_set_dash (frgba->private->gc, n_values, values, offset);
 	gnome_print_setdash (GNOME_PRINT_CONTEXT (frgba->private->meta), n_values, values, offset);
 	return gnome_print_setdash (frgba->private->context, n_values, values, offset);
 }
@@ -696,7 +634,6 @@ gpf_setfont (GnomePrintContext * pc, GnomeFont * font)
 
 	frgba = GNOME_PRINT_FRGBA (pc);
 
-	gp_gc_set_font (frgba->private->gc, font);
 	gnome_print_setfont (GNOME_PRINT_CONTEXT (frgba->private->meta), font);
 	return gnome_print_setfont (frgba->private->context, font);
 }
@@ -716,31 +653,12 @@ gpf_concat (GnomePrintContext * pc, const gdouble matrix[6])
 
 	frgba = GNOME_PRINT_FRGBA (pc);
 
-	gp_gc_concat (frgba->private->gc, matrix);
 	gnome_print_concat (GNOME_PRINT_CONTEXT (frgba->private->meta), matrix);
 	return gnome_print_concat (frgba->private->context, matrix);
 }
 
-#ifdef SETMATRIX_DEPRECATED
 static gint
-gpf_setmatrix (GnomePrintContext * pc, const gdouble matrix[6])
-{
-	GnomePrintFRGBA * frgba;
-
-	g_return_val_if_fail (pc != NULL, 0);
-	g_return_val_if_fail (GNOME_IS_PRINT_FRGBA (pc), 0);
-	g_return_val_if_fail (matrix != NULL, 0);
-
-	frgba = GNOME_PRINT_FRGBA (pc);
-
-	gp_gc_setmatrix (frgba->private->gc, matrix);
-	gnome_print_setmatrix (GNOME_PRINT_CONTEXT (frgba->private->meta), matrix);
-	return gnome_print_setmatrix (frgba->private->context, matrix);
-}
-#endif
-
-static gint
-gpf_clip (GnomePrintContext * pc)
+gpf_clip (GnomePrintContext * pc, ArtWindRule rule)
 {
 	GnomePrintFRGBA * frgba;
 
@@ -749,24 +667,15 @@ gpf_clip (GnomePrintContext * pc)
 
 	frgba = GNOME_PRINT_FRGBA (pc);
 
-	gp_gc_clip (frgba->private->gc);
-	gnome_print_clip (GNOME_PRINT_CONTEXT (frgba->private->meta));
-	return gnome_print_clip (frgba->private->context);
-}
-
-static gint
-gpf_eoclip (GnomePrintContext * pc)
-{
-	GnomePrintFRGBA * frgba;
-
-	g_return_val_if_fail (pc != NULL, 0);
-	g_return_val_if_fail (GNOME_IS_PRINT_FRGBA (pc), 0);
-
-	frgba = GNOME_PRINT_FRGBA (pc);
-
-	gp_gc_eoclip (frgba->private->gc);
-	gnome_print_eoclip (GNOME_PRINT_CONTEXT (frgba->private->meta));
-	return gnome_print_eoclip (frgba->private->context);
+	if (rule == ART_WIND_RULE_NONZERO) {
+		gp_gc_clip (pc->gc);
+		gnome_print_clip (GNOME_PRINT_CONTEXT (frgba->private->meta));
+		return gnome_print_clip (frgba->private->context);
+	} else {
+		gp_gc_eoclip (pc->gc);
+		gnome_print_eoclip (GNOME_PRINT_CONTEXT (frgba->private->meta));
+		return gnome_print_eoclip (frgba->private->context);
+	}
 }
 
 /*
@@ -802,7 +711,6 @@ gpf_showpage (GnomePrintContext * pc)
 	/* Create fresh meta */
 	frgba->private->meta = gnome_print_meta_new ();
 	/* Reset graphic context */
-	gp_gc_reset (frgba->private->gc);
 	return gnome_print_showpage (frgba->private->context);
 }
 
@@ -825,7 +733,6 @@ GnomePrintContext *
 gnome_print_frgba_new (GnomePrintContext * context)
 {
 	GnomePrintFRGBA * frgba;
-	GPGC * gc;
 	GnomePrintMeta * meta;
 
 	g_return_val_if_fail (context != NULL, NULL);
@@ -834,16 +741,12 @@ gnome_print_frgba_new (GnomePrintContext * context)
 	frgba = gtk_type_new (GNOME_TYPE_PRINT_FRGBA);
 	g_assert (frgba != NULL);
 
-	gc = gp_gc_new ();
-	g_assert (gc != NULL);
-
 	meta = gnome_print_meta_new ();
 	g_assert (meta != NULL);
 
 	frgba->private->context = context;
 	gtk_object_ref (GTK_OBJECT (context));
 
-	frgba->private->gc = gc;
 	frgba->private->meta = meta;
 
 	return GNOME_PRINT_CONTEXT (frgba);
@@ -913,7 +816,7 @@ gpf_render_buf (GnomePrintFRGBA * frgba, ArtDRect * box)
 	gtk_object_unref (GTK_OBJECT (gpr));
 
 	gnome_print_gsave (fp->context);
-	art_affine_invert (mtc, gp_gc_get_ctm (fp->gc));
+	art_affine_invert (mtc, gp_gc_get_ctm (GNOME_PRINT_CONTEXT (frgba)->gc));
 	gnome_print_concat (fp->context, mtc);
 	gnome_print_translate (fp->context, box->x0, box->y0);
 	gnome_print_scale (fp->context, (box->x1 - box->x0), (box->y1 - box->y0));
