@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 5; indent-tabs-mode: t; c-basic-offset: 5 -*- */
 /*
  *  Copyright (C) 2000 Helix Code Inc.
  *
@@ -61,11 +62,8 @@ typedef enum {
 	MODE_ZOOM_OUT
 } PreviewMode;
 
-struct _GnomePrintMasterPreview {
-	GnomeApp app;
-
-	void *priv;		/* private data */
 
+struct _GnomePrintMasterPreviewPrivate {
 	GnomePrintMaster *master; /* what drives us */
 
 	/*
@@ -83,13 +81,6 @@ struct _GnomePrintMasterPreview {
 	int                  current_page;
 	int                  pagecount;
 
-};
-
-struct _GnomePrintMasterPreviewClass {
-	GnomeAppClass parent_class;
-};
-
-struct _Private {
 	/* the output print context, a gnomeprintpreview */
 	GnomePrintPreview *output;
 
@@ -111,8 +102,7 @@ struct _Private {
 	gdouble              width, height;
 };
 
-static void preview_zoom_fit_cmd(GtkToggleButton *t, GnomePrintMasterPreview *pmp);
-
+#define Private GnomePrintMasterPreviewPrivate
 
 /*
  * Padding in points around the simulated page
@@ -124,7 +114,7 @@ render_page (GnomePrintMasterPreview *pmp, int page)
 {
 	const GnomePaper *paper;
 	const char *paper_name;
-	struct _Private *pp = pmp->priv;
+	Private *pp = pmp->priv;
 
 	gtk_object_unref (GTK_OBJECT (pp->output));
 	pp->output = NULL;
@@ -132,15 +122,15 @@ render_page (GnomePrintMasterPreview *pmp, int page)
 	/*
 	 * Create the preview printing context
 	 */
-	paper = pmp->master->paper;
+	paper = pp->master->paper;
 	paper_name = gnome_paper_name (paper);
-	pp->output = (GnomePrintPreview *)gnome_print_preview_new (pmp->canvas, paper_name);
+	pp->output = (GnomePrintPreview *)gnome_print_preview_new (pp->canvas, paper_name);
 
 	/*
 	 * Reset scrolling region always
 	 */
 	gnome_canvas_set_scroll_region (
-		pmp->canvas,
+		pp->canvas,
 		0 - PAGE_PAD,
 		0 - PAGE_PAD,
 		pp->width + PAGE_PAD,
@@ -156,23 +146,25 @@ render_page (GnomePrintMasterPreview *pmp, int page)
 		gnome_print_concat (GNOME_PRINT_CONTEXT (pp->output), affine);
 	}
 
-	gnome_print_meta_render_from_object_page (GNOME_PRINT_CONTEXT(pp->output),
-						  GNOME_PRINT_META (gnome_print_master_get_context(pmp->master)),
-						  page);
+	gnome_print_meta_render_from_object_page (
+		GNOME_PRINT_CONTEXT(pp->output),
+		GNOME_PRINT_META (gnome_print_master_get_context(pp->master)),
+		page);
 }
 
 static void
 goto_page (GnomePrintMasterPreview *pmp, int page)
 {
-	char text[4 * sizeof (int)];
-
-	if (page == pmp->current_page)
-		return;
-
-	pmp->current_page = page;
+	char text [4 * sizeof (int)];
+	Private *pp = pmp->priv;
 
 	sprintf (text, "%d", page + 1);
-	gtk_entry_set_text (GTK_ENTRY (pmp->page_entry), text);
+	gtk_entry_set_text (GTK_ENTRY (pp->page_entry), text);
+	
+	if (page == pp->current_page)
+		return;
+
+	pp->current_page = page;
 
 	render_page (pmp, page);
 }
@@ -180,6 +172,7 @@ goto_page (GnomePrintMasterPreview *pmp, int page)
 static void
 change_page_cmd (GtkEntry *entry, GnomePrintMasterPreview *pmp)
 {
+	Private *pp = pmp->priv;
 	char *text = gtk_entry_get_text (entry);
 	int p;
 
@@ -189,8 +182,8 @@ change_page_cmd (GtkEntry *entry, GnomePrintMasterPreview *pmp)
 		goto_page (pmp, 0);
 		return;
 	}
-	if (p > pmp->pagecount){
-		goto_page (pmp, pmp->pagecount-1);
+	if (p > pp->pagecount - 1){
+		goto_page (pmp, pp->pagecount-1);
 		return;
 	}
 	goto_page (pmp, p);
@@ -200,15 +193,16 @@ static void
 do_zoom (GnomePrintMasterPreview *pmp, int factor)
 {
 	double zoom;
-
+	Private *pp = pmp->priv;
+	
 	if (factor > 0)
-		zoom = pmp->canvas->pixels_per_unit * 1.25;
+		zoom = pp->canvas->pixels_per_unit * 1.25;
 	else if (factor < 0)
-		zoom = pmp->canvas->pixels_per_unit / 1.25;
+		zoom = pp->canvas->pixels_per_unit / 1.25;
 	else
 		zoom = 1.0;
 
-	gnome_canvas_set_pixels_per_unit (pmp->canvas, zoom);
+	gnome_canvas_set_pixels_per_unit (pp->canvas, zoom);
 }
 
 /* Button press handler for the print preview canvas */
@@ -216,7 +210,7 @@ static gint
 preview_canvas_button_press (GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
 	GnomePrintMasterPreview *pmp = data;
-	struct _Private *pp = pmp->priv;
+	Private *pp = pmp->priv;
 	int retval;
 
 	retval = FALSE;
@@ -255,7 +249,7 @@ preview_canvas_button_press (GtkWidget *widget, GdkEventButton *event, gpointer
 static void
 drag_to (GnomePrintMasterPreview *pmp, int x, int y)
 {
-	struct _Private *pp = pmp->priv;
+	Private *pp = pmp->priv;
 	int dx, dy;
 
 	dx = pp->drag_anchor_x - x;
@@ -269,7 +263,7 @@ drag_to (GnomePrintMasterPreview *pmp, int x, int y)
 	 *
 	 * This will be resolved in the canvas.
 	 */
-	gnome_canvas_scroll_to (pmp->canvas, pp->drag_ofs_x + dx, pp->drag_ofs_y + dy);
+	gnome_canvas_scroll_to (pp->canvas, pp->drag_ofs_x + dx, pp->drag_ofs_y + dy);
 }
 
 /* Button release handler for the print preview canvas */
@@ -277,7 +271,7 @@ static gint
 preview_canvas_button_release (GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
 	GnomePrintMasterPreview *pmp = data;
-	struct _Private *pp = pmp->priv;
+	Private *pp = pmp->priv;
 	int retval;
 
 	retval = FALSE;
@@ -306,7 +300,7 @@ static gint
 preview_canvas_motion (GtkWidget *widget, GdkEventMotion *event, gpointer data)
 {
 	GnomePrintMasterPreview *pmp = data;
-	struct _Private *pp = pmp->priv;
+	Private *pp = pmp->priv;
 	int retval;
 	gint x, y;
 	GdkModifierType mods;
@@ -343,7 +337,9 @@ preview_close_cmd (void *unused, GnomePrintMasterPreview *pmp)
 static void
 preview_file_print_cmd (void *unused, GnomePrintMasterPreview *pmp)
 {
-	gnome_print_master_print(pmp->master);
+	Private *pp = pmp->priv;
+	
+	gnome_print_master_print (pp->master);
 	/* should we clean ourselves up now? */
 }
 
@@ -356,9 +352,10 @@ preview_first_page_cmd (void *unused, GnomePrintMasterPreview *pmp)
 static void
 preview_next_page_cmd (void *unused, GnomePrintMasterPreview *pmp)
 {
-	int current_page = pmp->current_page;
+	Private *pp = pmp->priv;
+	int current_page = pp->current_page;
 
-	if (current_page+2 > pmp->pagecount)
+	if (current_page+2 > pp->pagecount)
 		return;
 	goto_page (pmp, current_page+1);
 }
@@ -366,7 +363,8 @@ preview_next_page_cmd (void *unused, GnomePrintMasterPreview *pmp)
 static void
 preview_prev_page_cmd (void *unused, GnomePrintMasterPreview *pmp)
 {
-	int current_page = pmp->current_page;
+	Private *pp = pmp->priv;
+	int current_page = pp->current_page;
 
 	if (current_page < 1)
 		return;
@@ -376,92 +374,95 @@ preview_prev_page_cmd (void *unused, GnomePrintMasterPreview *pmp)
 static void
 preview_last_page_cmd (void *unused, GnomePrintMasterPreview *pmp)
 {
-	goto_page (pmp, pmp->pagecount-1);
+	Private *pp = pmp->priv;
+	
+	goto_page (pmp, pp->pagecount-1);
 }
 
 static void
 preview_zoom_in_cmd (GtkToggleButton *t, GnomePrintMasterPreview *pmp)
 {
-	do_zoom(pmp, 1);
+	do_zoom (pmp, 1);
 }
 
 static void
 preview_zoom_out_cmd (GtkToggleButton *t, GnomePrintMasterPreview *pmp)
 {
-	do_zoom(pmp, -1);
+	do_zoom (pmp, -1);
 }
 
 static void
 preview_zoom_fit_cmd (GtkToggleButton *t, GnomePrintMasterPreview *pmp)
 {
 	double zoomx, zoomy;
-	struct _Private *pp = pmp->priv;
-	int width = GTK_WIDGET (pmp->canvas)->allocation.width;
-	int height = GTK_WIDGET (pmp->canvas)->allocation.height;
+	Private *pp = pmp->priv;
+	int width = GTK_WIDGET (pp->canvas)->allocation.width;
+	int height = GTK_WIDGET (pp->canvas)->allocation.height;
 	
 	zoomx = width / (pp->width + 5.0 + PAGE_PAD);
 	zoomy = height / (pp->height + 5.0 + PAGE_PAD);
 	
 	if (zoomy < zoomx)
 		zoomx = zoomy;
-	gnome_canvas_set_pixels_per_unit (pmp->canvas, zoomx);
+	gnome_canvas_set_pixels_per_unit (pp->canvas, zoomx);
 }
 
 static void
 preview_zoom_fit_wide_cmd (GtkToggleButton *t, GnomePrintMasterPreview *pmp)
 {
 	double zoom;
-	struct _Private *pp = pmp->priv;
-	int width = GTK_WIDGET (pmp->canvas)->allocation.width;
+	Private *pp = pmp->priv;
+	int width = GTK_WIDGET (pp->canvas)->allocation.width;
 
 	/* include paper decoration */
 	zoom = width / (pp->width + 5.0 + PAGE_PAD);
-	gnome_canvas_set_pixels_per_unit (pmp->canvas, zoom);
+	gnome_canvas_set_pixels_per_unit (pp->canvas, zoom);
 }
 
 static void
 preview_zoom_fit_tall_cmd (GtkToggleButton *t, GnomePrintMasterPreview *pmp)
 {
 	double zoom;
-	struct _Private *pp = pmp->priv;
-	int height = GTK_WIDGET (pmp->canvas)->allocation.height;
+	Private *pp = pmp->priv;
+	int height = GTK_WIDGET (pp->canvas)->allocation.height;
 	
 	/* include paper decoration */
 	zoom = height / (pp->height + 5.0 + PAGE_PAD);
-	gnome_canvas_set_pixels_per_unit (pmp->canvas, zoom);
+	gnome_canvas_set_pixels_per_unit (pp->canvas, zoom);
 }
 
 static gint
 preview_canvas_key (GtkWidget *widget, GdkEventKey *event, void *data)
 {
 	GnomePrintMasterPreview *pmp = data;
+	Private *pp = pmp->priv;
 	int x,y;
 	int height, width;
 	int domove=0;
 
-	gnome_canvas_get_scroll_offsets (pmp->canvas, &x, &y);
-	height = GTK_WIDGET(pmp->canvas)->allocation.height;
-	width = GTK_WIDGET(pmp->canvas)->allocation.width;
+	gnome_canvas_get_scroll_offsets (pp->canvas, &x, &y);
+	height = GTK_WIDGET (pp->canvas)->allocation.height;
+	width = GTK_WIDGET (pp->canvas)->allocation.width;
 
 	switch (event->keyval) {
 	case '1':
-		preview_zoom_fit_cmd(0, pmp);
+		preview_zoom_fit_cmd (0, pmp);
 		break;
 	case '2':		/* as good a key as any? */
-		preview_zoom_fit_wide_cmd(0, pmp);
+		preview_zoom_fit_wide_cmd (0, pmp);
 		break;
 	case '3':
-		preview_zoom_fit_tall_cmd(0, pmp);
+		preview_zoom_fit_tall_cmd (0, pmp);
 		break;
 	case '+':
 	case '=':
 	case GDK_KP_Add:
-		do_zoom(pmp, 1);
+		do_zoom (pmp, 1);
 		break;
 	case '-':
 	case '_':
 	case GDK_KP_Subtract:
-		do_zoom(pmp, -1);
+		do_zoom (pmp, -1);
 		break;
 	case GDK_KP_Right:
 	case GDK_Right:
@@ -500,9 +501,9 @@ preview_canvas_key (GtkWidget *widget, GdkEventKey *event, void *data)
 	case GDK_BackSpace:
 	page_up:
 		if (y<=0) {
-			if (pmp->current_page>0) {
-				goto_page(pmp, pmp->current_page-1);
-				y = GTK_LAYOUT(pmp->canvas)->height-height;
+			if (pp->current_page>0) {
+				goto_page (pmp, pp->current_page-1);
+				y = GTK_LAYOUT (pp->canvas)->height-height;
 			}
 		} else {
 			y-=height;
@@ -513,9 +514,9 @@ preview_canvas_key (GtkWidget *widget, GdkEventKey *event, void *data)
 	case GDK_Page_Down:
 	case ' ':
 	page_down:
-		if (y>=GTK_LAYOUT(pmp->canvas)->height-height) {
-			if (pmp->current_page<pmp->pagecount-1) {
-				goto_page(pmp, pmp->current_page+1);
+		if (y>=GTK_LAYOUT (pp->canvas)->height-height) {
+			if (pp->current_page<pp->pagecount-1) {
+				goto_page (pmp, pp->current_page+1);
 				y=0;
 			}
 		} else {
@@ -531,7 +532,7 @@ preview_canvas_key (GtkWidget *widget, GdkEventKey *event, void *data)
 		break;
 	case GDK_KP_End:
 	case GDK_End:
-		goto_page (pmp, pmp->pagecount-1);
+		goto_page (pmp, pp->pagecount-1);
 		y=0;
 		domove=1;
 		break;
@@ -543,7 +544,7 @@ preview_canvas_key (GtkWidget *widget, GdkEventKey *event, void *data)
 	}
 
 	if (domove)
-		gnome_canvas_scroll_to (pmp->canvas, x, y);
+		gnome_canvas_scroll_to (pp->canvas, x, y);
 
 	gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "key_press_event");
 	return TRUE;
@@ -553,7 +554,7 @@ static void
 create_preview_canvas (GnomePrintMasterPreview *pmp)
 {
 	GtkWidget *box, *status;
-	struct _Private *pp = pmp->priv;
+	Private *pp = pmp->priv;
 	const GnomePaper *paper;
 	const char *paper_name;
 	GnomeCanvasItem *i;
@@ -561,38 +562,38 @@ create_preview_canvas (GnomePrintMasterPreview *pmp)
 	gtk_widget_push_colormap (gdk_rgb_get_cmap ());
 	gtk_widget_push_visual (gdk_rgb_get_visual ());
 
-	pmp->scrolled_window = gtk_scrolled_window_new (NULL, NULL);
-	pmp->canvas = GNOME_CANVAS (gnome_canvas_new_aa ());
-	gnome_canvas_set_pixels_per_unit (pmp->canvas, 1.0);
+	pp->scrolled_window = gtk_scrolled_window_new (NULL, NULL);
+	pp->canvas = GNOME_CANVAS (gnome_canvas_new_aa ());
+	gnome_canvas_set_pixels_per_unit (pp->canvas, 1.0);
 
-	gtk_signal_connect (GTK_OBJECT (pmp->canvas), "button_press_event",
+	gtk_signal_connect (GTK_OBJECT (pp->canvas), "button_press_event",
 			    GTK_SIGNAL_FUNC (preview_canvas_button_press),
 			    pmp);
-	gtk_signal_connect (GTK_OBJECT (pmp->canvas), "button_release_event",
+	gtk_signal_connect (GTK_OBJECT (pp->canvas), "button_release_event",
 			    GTK_SIGNAL_FUNC (preview_canvas_button_release),
 			    pmp);
-	gtk_signal_connect (GTK_OBJECT (pmp->canvas), "motion_notify_event",
+	gtk_signal_connect (GTK_OBJECT (pp->canvas), "motion_notify_event",
 			    GTK_SIGNAL_FUNC (preview_canvas_motion),
 			    pmp);
-	gtk_signal_connect (GTK_OBJECT (pmp->canvas), "key_press_event",
+	gtk_signal_connect (GTK_OBJECT (pp->canvas), "key_press_event",
 			    GTK_SIGNAL_FUNC (preview_canvas_key),
 			    pmp);
 
-	gtk_container_add (GTK_CONTAINER (pmp->scrolled_window), GTK_WIDGET (pmp->canvas));
+	gtk_container_add (GTK_CONTAINER (pp->scrolled_window), GTK_WIDGET (pp->canvas));
 
 	/*
 	 * Create the preview printing context
 	 */
-	paper = pmp->master->paper;
+	paper = pp->master->paper;
 	paper_name = gnome_paper_name (paper);
-	pp->output = (GnomePrintPreview *)gnome_print_preview_new (pmp->canvas, paper_name);
+	pp->output = (GnomePrintPreview *)gnome_print_preview_new (pp->canvas, paper_name);
 
 	/*
 	 * Now add some padding above and below and put a simulated
 	 * page on the background
 	 */
 	i = gnome_canvas_item_new (
-		GNOME_CANVAS_GROUP (gnome_canvas_root (pmp->canvas)),
+		GNOME_CANVAS_GROUP (gnome_canvas_root (pp->canvas)),
 		gnome_canvas_rect_get_type (),
 		"x1",   	 0.0,
 		"y1",   	 0.0,
@@ -604,7 +605,7 @@ create_preview_canvas (GnomePrintMasterPreview *pmp)
 		NULL);
 	gnome_canvas_item_lower_to_bottom (i);
 	i = gnome_canvas_item_new (
-		GNOME_CANVAS_GROUP (gnome_canvas_root (pmp->canvas)),
+		GNOME_CANVAS_GROUP (gnome_canvas_root (pp->canvas)),
 		gnome_canvas_rect_get_type (),
 		"x1",   	 3.0,
 		"y1",   	 3.0,
@@ -614,7 +615,7 @@ create_preview_canvas (GnomePrintMasterPreview *pmp)
 		NULL);
 	gnome_canvas_item_lower_to_bottom (i);
 	gnome_canvas_set_scroll_region (
-		pmp->canvas,
+		pp->canvas,
 		0 - PAGE_PAD,
 		0 - PAGE_PAD,
 		pp->width + PAGE_PAD,
@@ -622,20 +623,20 @@ create_preview_canvas (GnomePrintMasterPreview *pmp)
 
 	box = gtk_vbox_new (FALSE, 0);
 	status = gtk_hbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (status), gtk_label_new (_("Page: ")), FALSE, FALSE, 0);
-	pmp->page_entry = gtk_entry_new ();
-	gtk_widget_set_usize (pmp->page_entry, 40, 0);
-	gtk_signal_connect (GTK_OBJECT (pmp->page_entry), "activate", change_page_cmd, pmp);
-	gtk_box_pack_start (GTK_BOX (status), pmp->page_entry, FALSE, FALSE, 0);
-	pmp->last = gtk_label_new ("");
-	gtk_box_pack_start (GTK_BOX (status), pmp->last, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (status), gtk_label_new (_ ("Page: ")), FALSE, FALSE, 0);
+	pp->page_entry = gtk_entry_new ();
+	gtk_widget_set_usize (pp->page_entry, 40, 0);
+	gtk_signal_connect (GTK_OBJECT (pp->page_entry), "activate", change_page_cmd, pmp);
+	gtk_box_pack_start (GTK_BOX (status), pp->page_entry, FALSE, FALSE, 0);
+	pp->last = gtk_label_new ("");
+	gtk_box_pack_start (GTK_BOX (status), pp->last, FALSE, FALSE, 0);
 
 	gtk_box_pack_start (GTK_BOX (box), status, FALSE, FALSE, 3);
-	gtk_box_pack_start (GTK_BOX (box), pmp->scrolled_window, TRUE, TRUE, 0);
-	gnome_app_set_contents (GNOME_APP(pmp), box);
+	gtk_box_pack_start (GTK_BOX (box), pp->scrolled_window, TRUE, TRUE, 0);
+	gnome_app_set_contents (GNOME_APP (pmp), box);
 	gtk_widget_show_all (box);
 
-	gtk_widget_grab_focus (GTK_WIDGET (pmp->canvas));
+	gtk_widget_grab_focus (GTK_WIDGET (pp->canvas));
 
 	return;
 }
@@ -649,54 +650,54 @@ static GnomeUIInfo preview_file_menu [] = {
 
 static GnomeUIInfo preview_view_menu [] = {
 	GNOMEUIINFO_ITEM_STOCK (
-		N_("_First page"), N_("Shows the first page"),
+		N_ ("_First page"), N_ ("Shows the first page"),
 		preview_first_page_cmd, GNOME_STOCK_PIXMAP_FIRST),
 	GNOMEUIINFO_ITEM_STOCK (
-		N_("_Previous page"), N_("Shows the previous page"),
+		N_ ("_Previous page"), N_ ("Shows the previous page"),
 		preview_prev_page_cmd, GNOME_STOCK_PIXMAP_BACK),
 	GNOMEUIINFO_ITEM_STOCK (
-		N_("_Next page"), N_("Shows the next page"),
+		N_ ("_Next page"), N_ ("Shows the next page"),
 		preview_next_page_cmd, GNOME_STOCK_PIXMAP_FORWARD),
 	GNOMEUIINFO_ITEM_STOCK (
-		N_("_Last page"), N_("Shows the last page"),
+		N_ ("_Last page"), N_ ("Shows the last page"),
 		preview_last_page_cmd, GNOME_STOCK_PIXMAP_LAST),
 
 	GNOMEUIINFO_SEPARATOR,
 
-	{ GNOME_APP_UI_ITEM, N_("Zoom _in"), N_("Zooms in"), preview_zoom_in_cmd },
-	{ GNOME_APP_UI_ITEM, N_("Zoom _out"), N_("Zooms out"), preview_zoom_out_cmd },
+	{ GNOME_APP_UI_ITEM, N_ ("Zoom _in"), N_ ("Zooms in"), preview_zoom_in_cmd },
+	{ GNOME_APP_UI_ITEM, N_ ("Zoom _out"), N_ ("Zooms out"), preview_zoom_out_cmd },
 
 	GNOMEUIINFO_END
 };
 
 static GnomeUIInfo top_menu [] = {
 	GNOMEUIINFO_MENU_FILE_TREE (preview_file_menu),
-	{ GNOME_APP_UI_SUBTREE, N_("_View"), NULL, preview_view_menu },
+	{ GNOME_APP_UI_SUBTREE, N_ ("_View"), NULL, preview_view_menu },
 	GNOMEUIINFO_END
 };
 
 static GnomeUIInfo toolbar [] = {
 	GNOMEUIINFO_ITEM_STOCK (
-		N_("Print"), N_("Prints the current file"),
+		N_ ("Print"), N_ ("Prints the current file"),
 		preview_file_print_cmd, GNOME_STOCK_PIXMAP_PRINT),
 	GNOMEUIINFO_ITEM_STOCK (
-		N_("First"), N_("Shows the first page"),
+		N_ ("First"), N_ ("Shows the first page"),
 		preview_first_page_cmd, GNOME_STOCK_PIXMAP_FIRST),
 	GNOMEUIINFO_ITEM_STOCK (
-		N_("Back"), N_("Shows the previous page"),
+		N_ ("Back"), N_ ("Shows the previous page"),
 		preview_prev_page_cmd, GNOME_STOCK_PIXMAP_BACK),
 	GNOMEUIINFO_ITEM_STOCK (
-		N_("Next"), N_("Shows the next page"),
+		N_ ("Next"), N_ ("Shows the next page"),
 		preview_next_page_cmd, GNOME_STOCK_PIXMAP_FORWARD),
 	GNOMEUIINFO_ITEM_STOCK (
-		N_("Last"), N_("Shows the last page"),
+		N_ ("Last"), N_ ("Shows the last page"),
 		preview_last_page_cmd, GNOME_STOCK_PIXMAP_LAST),
 
-	GNOMEUIINFO_ITEM (N_("Zoom in"), N_("Zooms the page in"), preview_zoom_in_cmd, stock_zoom_in_xpm),
-	GNOMEUIINFO_ITEM (N_("Zoom out"), N_("Zooms the page out"), preview_zoom_out_cmd, stock_zoom_out_xpm),
-	GNOMEUIINFO_ITEM (N_("Fit"), N_("Zooms to fit the whole page"), preview_zoom_fit_cmd, stock_zoom_fit_xpm),
-	GNOMEUIINFO_ITEM (N_("Fit Wide"), N_("Zooms to fit the width of the page"), preview_zoom_fit_wide_cmd, stock_zoom_fit_wide_xpm),
-	GNOMEUIINFO_ITEM (N_("Fit Tall"), N_("Zooms to fit the height of the page"), preview_zoom_fit_tall_cmd, stock_zoom_fit_tall_xpm),
+	GNOMEUIINFO_ITEM (N_ ("Zoom in"), N_ ("Zooms the page in"), preview_zoom_in_cmd, stock_zoom_in_xpm),
+	GNOMEUIINFO_ITEM (N_ ("Zoom out"), N_ ("Zooms the page out"), preview_zoom_out_cmd, stock_zoom_out_xpm),
+	GNOMEUIINFO_ITEM (N_ ("Fit"), N_ ("Zooms to fit the whole page"), preview_zoom_fit_cmd, stock_zoom_fit_xpm),
+	GNOMEUIINFO_ITEM (N_ ("Fit Wide"), N_ ("Zooms to fit the width of the page"), preview_zoom_fit_wide_cmd, stock_zoom_fit_wide_xpm),
+	GNOMEUIINFO_ITEM (N_ ("Fit Tall"), N_ ("Zooms to fit the height of the page"), preview_zoom_fit_tall_cmd, stock_zoom_fit_tall_xpm),
 	GNOMEUIINFO_END
 };
 
@@ -706,13 +707,13 @@ create_toplevel (GnomePrintMasterPreview *pmp)
 	char *old_msg_locale;
 	gint width, height;
 	const GnomePaper *paper;
-	struct _Private *pp;
+	Private *pp;
 
 	g_return_if_fail (pmp != NULL);
 
 	pp = pmp->priv;
 
-	paper  = pmp->master->paper;
+	paper  = pp->master->paper;
 	if (pp->landscape) {
 		pp->height = gnome_paper_pswidth  (paper);
 		pp->width  = gnome_paper_psheight (paper);
@@ -735,12 +736,12 @@ create_toplevel (GnomePrintMasterPreview *pmp)
 
 	old_msg_locale = g_strdup (textdomain (NULL));
 	textdomain (PACKAGE);
-	gnome_app_create_menus_with_data (GNOME_APP(pmp), top_menu, pmp);
+	gnome_app_create_menus_with_data (GNOME_APP (pmp), top_menu, pmp);
 
 	pp->toolbar = g_malloc (sizeof (toolbar));
 	memcpy (pp->toolbar, toolbar, sizeof (toolbar));
 
-	gnome_app_create_toolbar_with_data (GNOME_APP(pmp), pp->toolbar, pmp);
+	gnome_app_create_toolbar_with_data (GNOME_APP (pmp), pp->toolbar, pmp);
 	textdomain (old_msg_locale);
 	g_free (old_msg_locale);
 }
@@ -764,10 +765,10 @@ gnome_print_master_preview_get_type (void)
 			"GnomePrintMasterPreview",
 			sizeof (GnomePrintMasterPreview),
 			sizeof (GnomePrintMasterPreviewClass),
-			(GtkClassInitFunc) gnome_print_master_preview_class_init,
-			(GtkObjectInitFunc) gnome_print_master_preview_init,
-			(GtkArgSetFunc) NULL,
-			(GtkArgGetFunc) NULL
+			 (GtkClassInitFunc) gnome_print_master_preview_class_init,
+			 (GtkObjectInitFunc) gnome_print_master_preview_init,
+			 (GtkArgSetFunc) NULL,
+			 (GtkArgGetFunc) NULL
 		};
 		
 		print_master_preview_type = gtk_type_unique (gnome_app_get_type (), &print_master_preview_info);
@@ -779,17 +780,17 @@ gnome_print_master_preview_get_type (void)
 static void
 gnome_print_master_preview_finalize (GtkObject *object)
 {
-	GnomePrintMasterPreview *pmp = GNOME_PRINT_MASTER_PREVIEW(object);
-	struct _Private *pp = pmp->priv;
+	GnomePrintMasterPreview *pmp = GNOME_PRINT_MASTER_PREVIEW (object);
+	Private *pp = pmp->priv;
 
 	if (pp->output != NULL)
 		gtk_object_unref (GTK_OBJECT (pp->output));
 	g_free (pp->toolbar);
+
+	if (pp->master != NULL)
+		gtk_object_unref (GTK_OBJECT (pp->master));
+
 	g_free (pp);
-
-	if (pmp->master != NULL)
-		gtk_object_unref(GTK_OBJECT(pmp->master));
-
 	GTK_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -807,10 +808,10 @@ gnome_print_master_preview_class_init (GnomePrintMasterPreviewClass *class)
 static void
 gnome_print_master_preview_init (GnomePrintMasterPreview *pmp)
 {
-	struct _Private *pp;
+	Private *pp;
 
-	pp = pmp->priv = g_malloc0(sizeof(*pp));
-	pmp->current_page = -1;
+	pp = pmp->priv = g_malloc0 (sizeof (*pp));
+	pp->current_page = -1;
 }
 
 /**
@@ -828,7 +829,7 @@ gnome_print_master_preview_new (GnomePrintMaster *gpm,
 				const char *title)
 {
 	return gnome_print_master_preview_new_with_orientation
-		(gpm, title, FALSE);
+		 (gpm, title, FALSE);
 }
 
 /**
@@ -848,32 +849,32 @@ gnome_print_master_preview_new_with_orientation (GnomePrintMaster *gpm,
 						 gboolean landscape)
 {
 	GnomePrintMasterPreview *pmp;
-	struct _Private *pp;
+	Private *pp;
 	char text[4 * sizeof (int)];
 
-	pmp = GNOME_PRINT_MASTER_PREVIEW (gtk_type_new(gnome_print_master_preview_get_type()));
+	pmp = GNOME_PRINT_MASTER_PREVIEW (gtk_type_new (gnome_print_master_preview_get_type ()));
 
-	gnome_app_construct(GNOME_APP(pmp), "preview_window", title);
+	gnome_app_construct (GNOME_APP (pmp), "preview_window", title);
 
 	pp = pmp->priv;
 	pp->landscape = landscape;
 
-	pmp->master = gpm;
-	gtk_object_ref(GTK_OBJECT(gpm));
+	pp->master = gpm;
+	gtk_object_ref (GTK_OBJECT (gpm));
 
 	create_toplevel (pmp);
 	create_preview_canvas (pmp);
 
 	/* this zooms to fit, once we know how big the window actually is */
-	gtk_signal_connect(GTK_OBJECT(pmp->canvas), "realize",
-			   GTK_SIGNAL_FUNC(preview_zoom_fit_cmd), pmp);
+	gtk_signal_connect (GTK_OBJECT (pp->canvas), "realize",
+			   GTK_SIGNAL_FUNC (preview_zoom_fit_cmd), pmp);
 
-	pmp->pagecount = gnome_print_master_get_pages(gpm);
+	pp->pagecount = gnome_print_master_get_pages (gpm);
 
 	goto_page (pmp, 0);
 	
-	sprintf(text, "/%d", pmp->pagecount);
-	gtk_label_set_text (GTK_LABEL (pmp->last), text);
+	sprintf (text, "/%d", pp->pagecount);
+	gtk_label_set_text (GTK_LABEL (pp->last), text);
 
 	return pmp;
 }
