@@ -59,6 +59,9 @@ gnome_print_master_get_type (void)
 	return print_master_type;
 }
 
+/* REmove */
+#include <libgnomeprint/gnome-printer-private.h>
+
 static void
 gnome_print_master_finalize (GtkObject *object)
 {
@@ -200,7 +203,7 @@ gnome_print_master_get_paper (const GnomePrintMaster *gpm)
 /**
  * gnome_print_master_set_printer:
  * @gpm: An initialised GnomePrintMaster.
- * @paper: Printer device to use.
+ * @printer: Printer device to use.
  * 
  * Sets physical printer used to print to.  If this is not set,
  * then a dialogue will be presented on each print.
@@ -302,7 +305,8 @@ gnome_print_master_print(GnomePrintMaster *gpm)
 	} else
 		gtk_object_ref (GTK_OBJECT (printer));
 
-	output = gnome_print_context_new_with_paper_size(printer, gnome_paper_name (gpm->paper));
+	output = gnome_print_context_new_with_paper_size (printer,
+											gnome_paper_name (gpm->paper));
 	if (output == NULL) {
 		gtk_object_unref (GTK_OBJECT (printer));
 		return -1;
