@@ -34,37 +34,17 @@ BEGIN_GNOME_DECLS
 #define GNOME_PRINT_MASTER_CLASS(klass) GTK_CHECK_CLASS_CAST (klass, gnome_print_master_get_type (), GnomePrintMasterClass)
 #define GNOME_IS_PRINT_MASTER(obj)      GTK_CHECK_TYPE (obj, gnome_print_master_get_type ())
 
-
 typedef struct _GnomePrintMaster      GnomePrintMaster;
 typedef struct _GnomePrintMasterClass GnomePrintMasterClass;
 
-struct _GnomePrintMaster {
-	GtkObject object;
-
-	void *priv;		/* private data */
-
-	GnomePrintContext *context; /* metafile context */
-
-	/* # copies info */
-	gint copies;
-	gboolean iscollate;
-
-	/* paper */
-	const GnomePaper *paper;
-
-	/* printer, if required */
-	GnomePrinter *printer;
-};
-
-struct _GnomePrintMasterClass {
-	GtkObjectClass parent_class;
-};
-
 guint		gnome_print_master_get_type	(void);
 GnomePrintMaster *gnome_print_master_new	(void);
 GnomePrintMaster *gnome_print_master_new_from_dialog (GnomePrintDialog *dialog);
 GnomePrintContext *gnome_print_master_get_context(GnomePrintMaster *gpm);
+
 void gnome_print_master_set_paper(GnomePrintMaster *gpm, const GnomePaper *printer);
+const GnomePaper * gnome_print_master_get_paper(const GnomePrintMaster *gpm);
+
 void gnome_print_master_set_printer(GnomePrintMaster *gpm, GnomePrinter *printer);
 void gnome_print_master_set_copies(GnomePrintMaster *gpm, int copies, gboolean iscollate);
 void gnome_print_master_close(GnomePrintMaster *gpm);
