@@ -1133,11 +1133,11 @@ do_render (GnomePrintContext *dest, const char *data, int size, int pages)
 			data = decode_int (data, &length);
 
 			if (length > 0) {
-				char *text;
+				char *text = g_malloc (length);
 
-				text = alloca (length);
 				data = decode_block (data, text, length);
 				gnome_print_show_sized (dest, text, length);
+				g_free (text);
 			}
 
 			break;
