@@ -2,15 +2,30 @@
 /*
  * gnome-print-pdf.c: A PDF driver for GnomePrint 
  *
- * Authors:
- *   Chema Celorio (chema@celorio.com)
+ * (C) 2000 Jose M Celorio
  *
- * (C) 2000 Jose M Celorio1528
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  * References :
  * 
  * [1] Portable Document Format Referece Manual, Version 1.3 (March 11, 1999)
  *
+ * Authors:
+ *   Chema Celorio (chema@celorio.com)
+ *
+ *
  */
 
 
@@ -35,7 +50,7 @@
 	
 #include "config.h"
 #include <unistd.h> /* For getpid () */
-#include <time.h>
+#include <time.h>   /* For time() */
 #include <gtk/gtk.h>
 #include <string.h>
 #include <math.h>
@@ -120,6 +135,7 @@ struct _GnomePrintPdfImage {
 	gint image_number;
 	gint object_number;
 	gint image_type;
+	gboolean compressed;
 };
 
 
@@ -256,7 +272,7 @@ static void gnome_print_pdf_init          (GnomePrintPdf *pdf);
 static void gnome_print_pdf_finalize      (GtkObject *object);
 
 gint gnome_print_pdf_write         (GnomePrintContext *pc, const char *format, ...);
-static void gnome_print_pdf_graphic_mode_set (GnomePrintPdf *pdf, gint mode);
+static gint gnome_print_pdf_graphic_mode_set (GnomePrintPdf *pdf, gint mode);
 
 
 /* -------------------------- PUBLIC FUNCTIONS ------------------------ */
@@ -357,26 +373,6 @@ gnome_print_pdf_add_bytes_written (GnomePrintPdf *pdf, gint bytes)
 	return 0;
 }
 
-/* -------------------------- END: PUBLIC FUNCTIONS ------------------------ */
-
-static guint
-gnome_print_pdf_object_number (GnomePrintContext *pc)
-{
-	GnomePrintPdf *pdf;
-	GnomePrintPdfObject *object;
-
-	debug (FALSE, "");
-	
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
-	pdf = GNOME_PRINT_PDF (pc);
-	g_return_val_if_fail (pdf != NULL, -1);
-	
-	object = g_new (GnomePrintPdfObject, 1);
-
-	pdf->objects = g_list_prepend (pdf->objects, object);
-
-	return ++pdf->object_number_last_used;
-}
 
 gint
 gnome_print_pdf_object_start (GnomePrintContext *pc, guint object_number)
@@ -434,6 +430,26 @@ gnome_print_pdf_object_end (GnomePrintContext *pc, guint object_number, guint do
 	return ret;
 }
 
+/* -------------------------- END: PUBLIC FUNCTIONS ------------------------ */
+static guint
+gnome_print_pdf_object_number (GnomePrintContext *pc)
+{
+	GnomePrintPdf *pdf;
+	GnomePrintPdfObject *object;
+
+	debug (FALSE, "");
+	
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	pdf = GNOME_PRINT_PDF (pc);
+	g_return_val_if_fail (pdf != NULL, -1);
+	
+	object = g_new (GnomePrintPdfObject, 1);
+
+	pdf->objects = g_list_prepend (pdf->objects, object);
+
+	return ++pdf->object_number_last_used;
+}
+
 static GnomePrintPdfGraphicState *
 gnome_print_pdf_graphic_state_current (GnomePrintPdf *pdf, gint dirtyfy)
 {
@@ -490,7 +506,7 @@ static gint
 gnome_print_pdf_page_write_contents (GnomePrintContext *pc, GnomePrintPdfPage *page)
 {
 	GnomePrintPdf *pdf;
-	gboolean compress = TRUE;
+	gboolean compress = FALSE;
 	gchar *compressed_stream = NULL;
 	gint ret = 0;
 	gint compressed_stream_length;
@@ -605,7 +621,7 @@ gnome_print_pdf_encoding (GnomePrintContext *pc, GnomePrintPdfFont *font_in)
 }
 
 
-/* FIXME : fix the '-' character issue */
+/* FIXME : fix the '-' character issue. Chema */
 static gint
 gnome_print_pdf_font_print_metrics (GnomePrintContext *pc, GnomePrintPdfFont *font_in)
 {
@@ -631,9 +647,14 @@ gnome_print_pdf_font_print_metrics (GnomePrintContext *pc, GnomePrintPdfFont *fo
 	nfonts = (nglyphs + 255) >> 8;
 	if (nfonts > 1) {
 		static gboolean warned = FALSE;
-		if (!warned)
-			g_warning ("Can't encode fonts with more than 1 page of glyphs");
+		if (!warned || TRUE)
+			g_warning ("\nCan't encode fonts with more than 1 page of glyphs for now.\n"
+								 "Encoding only the first page of glyphs (256). The font in \n"
+								 "question is :%s, which has %i glyphs. \n",
+								 gnome_font_face_get_name (face),
+								 gnome_font_face_get_num_glyphs (face));
 		warned = TRUE;
+		nfonts = 1;
 	}
 
 	if (nglyphs > 256)
@@ -647,11 +668,7 @@ gnome_print_pdf_font_print_metrics (GnomePrintContext *pc, GnomePrintPdfFont *fo
 																 nglyphs - 1);
 
 	
-#if 0	
 	for (i = 0; i < nfonts; i++) 
-#else
-		i = 0;
-#endif
 	{
 		gint col = 0;
 		for (j = 1; j < nglyphs; j++) {
@@ -772,7 +789,7 @@ gnome_print_pdf_font_print_descriptor (GnomePrintContext *pc,
 	ascent = (gint) gnome_font_face_get_ascender (face);
 	descent = (gint) gnome_font_face_get_descender (face);
 	italic_angle = (gint) gnome_font_face_get_italics_angle (face);
-	rect = gnome_font_face_get_font_bbox (face);
+	rect = gnome_font_face_get_stdbbox (face);
 	capheight = (gint) gnome_font_face_get_capheight (face);
 	flags = gnome_font_face_get_pdf_flags (face);
 	stemv = gnome_font_face_get_stemv (face);
@@ -929,10 +946,16 @@ gnome_print_pdf_images (GnomePrintContext *pc, GnomePrintPdfPage *page)
 																	 image->width,
 																	 image->height,
 																	 image_type_text);
-		ret += gnome_print_pdf_write  (pc,
-																	 "/Length %i" EOL
-																	 "/Filter /FlateDecode" EOL,
-																	 image->data_length + strlen (EOL));
+		if (image->compressed)
+			ret += gnome_print_pdf_write  (pc,
+																		 "/Length %i" EOL
+																		 "/Filter /FlateDecode" EOL,
+																		 image->data_length + strlen (EOL));
+		else
+			ret += gnome_print_pdf_write  (pc,
+																		 "/Length %i" EOL
+																		 "/Filter /ASCIIHexDecode" EOL, 
+																		 image->data_length + strlen (EOL));			
 		
 		ret += gnome_print_pdf_write  (pc,
 																	 ">>" EOL
@@ -1413,7 +1436,7 @@ gnome_print_pdf_pages (GnomePrintContext *pc)
 	}
 
 	g_return_val_if_fail (pdf->paper != NULL, -1);
-	paper_width  = gnome_paper_pswidth  (pdf->paper);
+	paper_width  = gnome_paper_pswidth (pdf->paper);
 	paper_height = gnome_paper_psheight (pdf->paper);
 	ret += gnome_print_pdf_write (pc,
 																"]" EOL
@@ -1710,36 +1733,41 @@ gnome_print_pdf_setrgbcolor (GnomePrintContext *pc,
 	return 0;
 }
 
-static void
+static gint
 gnome_print_pdf_graphic_mode_set (GnomePrintPdf *pdf, gint mode)
 {
+	gint ret = 0;
 
 	debug (FALSE, "");
 
 	if (pdf->graphics_mode == mode)
-		return;
+		return 0;
 
 	switch (mode) {
 	case PDF_GRAPHIC_MODE_GRAPHICS:
 		if (pdf->graphics_mode == PDF_GRAPHIC_MODE_TEXT)
-			gnome_print_pdf_write_content (pdf, "ET" EOL);
+			ret += gnome_print_pdf_write_content (pdf, "ET" EOL);
 		break;
 	case PDF_GRAPHIC_MODE_TEXT:
-		gnome_print_pdf_write_content (pdf, "BT" EOL);
+		ret += gnome_print_pdf_write_content (pdf, "BT" EOL);
 		break;
 	case PDF_GRAPHIC_MODE_UNDEFINED:
 		gnome_print_pdf_error (FALSE, "GRAPHIC_MODE undefined\n");
+		ret = -1;
 		break;
 	default:
 		gnome_print_pdf_error (FALSE, "mem-problems\n");
+		ret = -1;
 		g_assert_not_reached ();
 	}
 
 	pdf->graphics_mode = mode;
+
+	return ret;
 }			
 
 static gint
-gnome_print_pdf_setlinewidth (GnomePrintContext *pc, double linewidth)
+gnome_print_pdf_setlinewidth (GnomePrintContext *pc, double width)
 {
 	GnomePrintPdf *pdf;
 	GnomePrintPdfGraphicState *gs;
@@ -1750,13 +1778,15 @@ gnome_print_pdf_setlinewidth (GnomePrintContext *pc, double linewidth)
 	pdf = GNOME_PRINT_PDF (pc);
 	g_return_val_if_fail (pdf != NULL, -1);
 
-	if (linewidth < 0)
-		return gnome_print_pdf_error (FALSE, "setlinewidth, invalid parameter range %g", linewidth);
+	if (width < 0)
+		return gnome_print_pdf_error (FALSE, "setlinewidth, invalid parameter range %g", width);
 	
 	gs = gnome_print_pdf_graphic_state_current (pdf, TRUE);
 
-	gs->linewidth = linewidth;
-
+	gs->linewidth = (fabs (width * gs->ctm[0]) +
+									 fabs (width * gs->ctm[1]) +
+									 fabs (width * gs->ctm[2]) +
+									 fabs (width * gs->ctm[3])) / 2;	
 	return 0;
 }
 
@@ -2237,6 +2267,14 @@ gnome_print_pdf_gsave (GnomePrintContext *pc)
 	pdf = GNOME_PRINT_PDF (pc);
 	g_return_val_if_fail (pdf != NULL, -1);
 
+	/* This is messy, but I don't want to play arround with it before
+		 Gnome 1.4 is out. I don't think there is anything wrong. But since
+		 we are setting the mode to graphics before gsave we are always going
+	   to do gsaves and grestores in GRAPHICS mode, for that reason we don't
+	   need to save the graphics_mode, we know it to be MODE_GRAPHICS. Just clean
+	   it after 1.4 Chema. */
+	gnome_print_pdf_graphic_mode_set (pdf, PDF_GRAPHIC_MODE_GRAPHICS);
+	
 	gs_push_me     = gnome_print_pdf_graphic_state_duplicate (pdf->graphic_state);
 	gs_push_me_set = gnome_print_pdf_graphic_state_duplicate (pdf->graphic_state_set);
 
@@ -2246,6 +2284,11 @@ gnome_print_pdf_gsave (GnomePrintContext *pc)
 
 	pdf->gsave_level_number++;
 
+	if (pdf->gsave_level_number == pdf->gsave_level_max) {
+		pdf->gsave_stack = g_realloc (pdf->gsave_stack, sizeof (GnomePrintPdfGsave) *
+																	(pdf->gsave_level_max += GNOME_PRINT_NUMBER_OF_ELEMENTS_GROW));
+	}
+	
 	ret += gnome_print_pdf_write_content (pdf, "q" EOL);
 
 	return ret;
@@ -2264,6 +2307,13 @@ gnome_print_pdf_grestore (GnomePrintContext *pc)
 	pdf = GNOME_PRINT_PDF (pc);
 	g_return_val_if_fail (pdf != NULL, -1);
 
+	/* * * * * * * * * * */
+#if 0	
+	mode = pdf->gsave_stack [pdf->gsave_level_number].graphics_mode;
+	gnome_print_pdf_graphic_mode_set (pdf, PDF_GRAPHIC_MODE_GRAPHICS);
+#endif	
+	/* * * * * * * * * * */
+	
 	pdf->gsave_level_number--;
 
 	if (pdf->gsave_level_number < 0) {
@@ -2276,8 +2326,10 @@ gnome_print_pdf_grestore (GnomePrintContext *pc)
 	pdf->graphic_state_set = pdf->gsave_stack [pdf->gsave_level_number].graphic_state_set;
 	pdf->graphic_state     = pdf->gsave_stack [pdf->gsave_level_number].graphic_state;
 
+	/* * * * * * * * * * */
 	mode = pdf->gsave_stack [pdf->gsave_level_number].graphics_mode;
 	gnome_print_pdf_graphic_mode_set (pdf, mode);
+	/* * * * * * * * * * */
 
 	ret += gnome_print_pdf_write_content (pdf, "Q" EOL);
 
@@ -2285,11 +2337,12 @@ gnome_print_pdf_grestore (GnomePrintContext *pc)
 }
 
 
-static inline gboolean
-gp_zero_clean (double cleanme)
+static gdouble
+gp_zero_clean (gdouble cleanme)
 {
-	if (abs(cleanme) > 0.00000001)
+	if (abs(cleanme) > 0.00000001) {
 		return cleanme;
+	}
 	return 0.0;
 }
 
@@ -2304,18 +2357,18 @@ gnome_print_pdf_add_glyph_to_font (GnomePrintPdfFont *pdf_font, const gchar *gly
 	g_return_val_if_fail (glyph_name != NULL, FALSE);
 
 	for (n = 0; n < pdf_font->glyphs_num; n++) {
-		if (!strcmp (glyph_name, pdf_font->glyphs[n]))
+		if (!strcmp (glyph_name, pdf_font->glyphs[n].name))
 			break;
 	}
 
 	if (n == pdf_font->glyphs_num) {
 		if (pdf_font->glyphs_num == pdf_font->glyphs_max)
-			pdf_font->glyphs = g_realloc (pdf_font->glyphs, sizeof (gchar *) *
+			pdf_font->glyphs = g_realloc (pdf_font->glyphs, sizeof (T1Glyph) *
 																		(pdf_font->glyphs_max +=
 																		 GNOME_PRINT_NUMBER_OF_ELEMENTS_GROW));
-		pdf_font->glyphs [pdf_font->glyphs_num++] = g_strdup (glyph_name);
+		pdf_font->glyphs[pdf_font->glyphs_num++].name = g_strdup (glyph_name);
 	}
-
+	
 	return TRUE;
 }
 	
@@ -2355,7 +2408,7 @@ gnome_print_pdf_show_sized (GnomePrintContext *pc, const char *text, int bytes)
 		gnome_print_pdf_error (FALSE, "show, fontname or fontsize not defined.");
 		return -1;
 	}
-	
+
 	val_1 = gp_zero_clean (gs->font_size * gs->ctm[0]);
 	val_2 = gp_zero_clean (gs->font_size * gs->ctm[1]);
 	val_3 = gp_zero_clean (gs->font_size * gs->ctm[2]);
@@ -2363,7 +2416,7 @@ gnome_print_pdf_show_sized (GnomePrintContext *pc, const char *text, int bytes)
 
 	gp_path_currentpoint (gs->current_path, &point);
 
-	gnome_print_pdf_graphic_mode_set (pdf, PDF_GRAPHIC_MODE_TEXT);
+	ret += gnome_print_pdf_graphic_mode_set (pdf, PDF_GRAPHIC_MODE_TEXT);
 
 	ret += gnome_print_pdf_graphic_state_set_font (pdf);
 
@@ -2373,7 +2426,7 @@ gnome_print_pdf_show_sized (GnomePrintContext *pc, const char *text, int bytes)
 	g_return_val_if_fail (GNOME_IS_FONT_FACE (face), -1);
 
 	ret += gnome_print_pdf_write_content (pdf,
-																				"%g %g %g %g %g %g Tm" EOL,
+																				"%f %f %f %f %f %f Tm" EOL,
 																				val_1,
 																				val_2,
 																				val_3,
@@ -2586,15 +2639,50 @@ gnome_print_pdf_curveto (GnomePrintContext *pc,
 	return 0;	
 }
 
+static gboolean
+gnome_print_pdf_font_insert (GnomePrintPdf *pdf,
+														 GnomeFont *gnome_font,
+														 const gchar *font_name,
+														 gboolean is_basic_14)
+{
+	GnomePrintPdfFont *font;
+
+	debug (FALSE, "");
+	
+	g_return_val_if_fail (GNOME_IS_PRINT_PDF (pdf), FALSE);
+	g_return_val_if_fail (GNOME_IS_FONT (gnome_font), FALSE);
+	g_return_val_if_fail (font_name != NULL, FALSE);
+	
+	if (pdf->fonts_number == pdf->fonts_max)
+		pdf->fonts = g_realloc (pdf->fonts, sizeof (GnomePrintPdfFont) *
+													 (pdf->fonts_max += GNOME_PRINT_NUMBER_OF_ELEMENTS_GROW));
+
+	font = &pdf->fonts[pdf->fonts_number++];
+	font->font_number   = pdf->fonts_number;
+	font->gnome_font    = gnome_font;
+	font->font_name     = g_strdup (font_name);
+	font->object_number = 0;
+	font->object_number_descriptor = 5890990;
+	font->object_number_pfb        = 2931915;
+	font->is_basic_14   = is_basic_14;
+
+	font->glyphs_max    = GNOME_PRINT_NUMBER_OF_ELEMENTS;
+	font->glyphs_num    = 0;
+	font->glyphs        = g_new (T1Glyph, font->glyphs_max);
+
+	gtk_object_ref (GTK_OBJECT(gnome_font));
+	
+	return TRUE;
+}
+
 static gint
 gnome_print_pdf_get_font_number (GnomePrintContext *pc,
 																 GnomeFont *gnome_font,
 																 gboolean is_basic_14)
 {
 	GnomePrintPdf *pdf;
-	GnomePrintPdfFont *font;
 	gint n;
-	gchar *font_name;
+	const gchar *font_name;
 
 	debug (FALSE, "");
 
@@ -2603,41 +2691,21 @@ gnome_print_pdf_get_font_number (GnomePrintContext *pc,
 	g_return_val_if_fail (pdf != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_FONT (gnome_font), -1);
 
-	font_name = g_strdup (gnome_font_get_ps_name (gnome_font));
+	font_name = gnome_font_get_ps_name (gnome_font);
 
 	for (n = 0; n < pdf->fonts_number; n++)
 		if (!strcmp (font_name, pdf->fonts[n].font_name))
 			break;
 
-	if (n != pdf->fonts_number) {
-		g_free (font_name);
+	if (n != pdf->fonts_number)
 		return n;
-	}
 
-	if (pdf->fonts_number == pdf->fonts_max)
-		pdf->fonts = g_realloc (pdf->fonts, sizeof (GnomePrintPdfFont) *
-													 (pdf->fonts_max += GNOME_PRINT_NUMBER_OF_ELEMENTS_GROW));
+	gnome_print_pdf_font_insert (pdf, gnome_font, font_name, is_basic_14);
 
-	font = &pdf->fonts[pdf->fonts_number++];
-	font->font_number   = pdf->fonts_number*2;
-	font->gnome_font    = gnome_font;
-	font->font_name     = font_name;
-	font->object_number = 0;
-	font->object_number_descriptor = 5890990;
-	font->object_number_pfb        = 2931915;
-	font->is_basic_14   = is_basic_14;
-	font->lowest        = 255; 
-	font->highest       = 0;
-
-	font->glyphs_max    = GNOME_PRINT_NUMBER_OF_ELEMENTS;
-	font->glyphs_num    = 0;
-	font->glyphs        = g_new (gchar *, font->glyphs_max);
-
-	gtk_object_ref (GTK_OBJECT(gnome_font));
-	
 	return pdf->fonts_number-1;
 }
 
+
 static gint
 gnome_print_pdf_setfont (GnomePrintContext *pc, GnomeFont *font)
 {
@@ -2707,8 +2775,7 @@ gnome_print_pdf_strokepath (GnomePrintContext *pc)
 	if (warned)
 		return 0;
 	
-	g_warning ("Strokepath is not supported. It migth get deprecated\n"
-						 "and replaced with gnome_print_stroke_and_fill");
+	g_warning ("Strokepath is not supported. It migth get deprecated in the future\n");
 	warned = TRUE;
 	
   return 0;
@@ -2796,7 +2863,7 @@ gnome_print_pdf_closepath (GnomePrintContext *pc)
 static gint
 gnome_print_pdf_image_load (GnomePrintPdf *pdf, gchar *data, gint width, gint height,
 														gint rowstride, gint bytes_per_pixel, gint image_type,
-														gint data_length)
+														gint data_length, gboolean compressed)
 {
 	GnomePrintPdfImage *image;
 	GnomePrintPdfPage *page;
@@ -2821,6 +2888,7 @@ gnome_print_pdf_image_load (GnomePrintPdf *pdf, gchar *data, gint width, gint he
 	image->image_number = page->images_number;
 	image->object_number = gnome_print_pdf_object_number (GNOME_PRINT_CONTEXT(pdf));
 	image->image_type = image_type;
+	image->compressed = compressed;
 
 	return image->image_number;
 }
@@ -2834,12 +2902,13 @@ gnome_print_pdf_image_compressed (GnomePrintContext *pc,
 																	gint bytes_per_pixel,
 																	gint image_type)
 {
-	GnomePrintPdf *pdf;
 	GnomePrintPdfGraphicState *gs;
+	GnomePrintPdf *pdf;
+	gboolean compress = FALSE; /* FAlse is NOT working */
+	gchar *image_stream;
 	gint ret = 0;
 	gint image_number;
-	gchar *image_stream;
-	gint   image_stream_size;
+	gint image_stream_size;
 
 	g_return_val_if_fail (GNOME_IS_PRINT_PDF(pc), -1);
 	pdf = GNOME_PRINT_PDF (pc);
@@ -2849,29 +2918,33 @@ gnome_print_pdf_image_compressed (GnomePrintContext *pc,
 
 	gs = pdf->graphic_state;
 
+	gnome_print_pdf_graphic_mode_set (pdf, PDF_GRAPHIC_MODE_GRAPHICS);
+	
 	ret += gnome_print_pdf_write_content (pdf, "%g %g %g %g %g %g cm" EOL,
 																				gs->ctm[0], gs->ctm[1],
 																				gs->ctm[2],	gs->ctm[3],
 																				gs->ctm[4], gs->ctm[5]);
-#if 0	
-	image_stream_size = gnome_print_encode_ascii85_wcs (width * height * bytes_per_pixel);
-#else	
-	image_stream_size = gnome_print_encode_deflate_wcs (width * height * bytes_per_pixel);
-#endif	
+	ret += gnome_print_pdf_write_content (pdf, "0 0 m" EOL);
+	
+	if (compress)
+		image_stream_size = gnome_print_encode_deflate_wcs (width * height * bytes_per_pixel);
+	else
+		image_stream_size = gnome_print_encode_hex_wcs (width * height * bytes_per_pixel);
+	
 	image_stream      = g_new (gchar, image_stream_size);
 
-#if 0	
-		image_stream_size = gnome_print_encode_ascii85 (data, image_stream, width * height * bytes_per_pixel);
-#else	
+	if (compress)
 		image_stream_size = gnome_print_encode_deflate (data, image_stream,
-																									width * height * bytes_per_pixel,
-																									image_stream_size);
-#endif	
+																										width * height * bytes_per_pixel,
+																										image_stream_size);
+	else
+		image_stream_size = gnome_print_encode_hex (data, image_stream, width * height * bytes_per_pixel);
 
-	image_number      = gnome_print_pdf_image_load (pdf, image_stream,
-																									width, height,
-																									rowstride, bytes_per_pixel,
-																									image_type, image_stream_size);
+	image_number = gnome_print_pdf_image_load (pdf, image_stream,
+																						 width, height,
+																						 rowstride, bytes_per_pixel,
+																						 image_type, image_stream_size,
+																						 compress);
 
 	ret += gnome_print_pdf_write_content (pdf, "/Im%i Do" EOL,
 																				image_number);
@@ -2933,164 +3006,6 @@ gnome_print_pdf_rgbimage (GnomePrintContext *pc,
   return ret;
 }
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-/* - - - - - - - - - - - - - - - - - - - OLD STUFF NOT IMPLEMENTED  - - - - - - - - - - - - - - - - - - - */
-
-#ifdef WACHAMACA_HACK
-static gint
-gnome_print_pdf_setfont_raw (GnomePrintContext *pc, GnomeFontFace * face,
-			    double size)
-{
-  GnomePrintPdf *pdf;
-  char *pfa;
-  gint i;
-  const char *fontname;
-
-  g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
-  pdf = GNOME_PRINT_PDF (pc);
-
-  if (face == NULL)
-    return -1;
-
-  fontname = gnome_font_face_get_ps_name (face);
-  for (i = 0; i < pdf->n_builtins; i++)
-    if (!strcmp (fontname, pdf->builtins[i]))
-      break;
-
-  if (i == pdf->n_builtins)
-    {
-      fontname = gnome_font_unsized_get_glyph_name (face);
-      for (i = 0; i < pdf->n_fonts; i++)
-	if (!strcmp (fontname, pdf->fonts[i]))
-	  break;
-      if (i == pdf->n_fonts)
-	{
-	  pfa = gnome_font_face_get_pfa (face);
-	  if (pfa == NULL)
-	    return -1;
-	  
-	  if (gnome_print_pdf_write (pc, "%s", pfa) < 0)
-	    return -1;
-	  
-	  if (pdf->n_fonts == pdf->n_fonts_max)
-	    pdf->fonts = g_realloc (pdf->fonts, sizeof (char *) *
-															(pdf->n_fonts_max <<= 1));
-	  pdf->fonts[pdf->n_fonts++] = g_strdup (fontname);
-		g_free (pfa);
-	  gnome_print_pdf_reencode_font (pc, fontname);
-	}
-    }
-  else
-    {
-      for (i = 0; i < pdf->n_fonts_reencoded; i++)
-        if (!strcmp (fontname, pdf->fonts_reencoded[i]))
-	  break;
-
-      if (i == pdf->n_fonts_reencoded)
-        {
-          if (pdf->n_fonts_reencoded == pdf->n_fonts_reencoded_max)
-	    pdf->fonts_reencoded = g_realloc (pdf->fonts_reencoded, sizeof (char *) *
-																				(pdf->n_fonts_reencoded_max <<= 1));
-	  pdf->fonts_reencoded[pdf->n_fonts_reencoded++] = g_strdup (fontname);
-          gnome_print_pdf_reencode_font (pc, fontname);
-	}
-    }
-
-  /* NOT always true because of grestore and gsave. Should use
-     a flag to set if grestore has been used **/  
-#if 0
-          if (pdf->current_font && strcmp (pdf->current_font, fontname) == 0 &&
-              pdf->current_font_size == size)
-                  return 0; /* this font is already set */
-#endif
-          
-  pdf->current_font = fontname;
-  pdf->current_font_size = size;
-
-  return gnome_print_pdf_write (pc,
-				      "/La-%s findfont %g scalefont setfont\n",
-				      fontname, size);
-}
-#endif 
-
-
 static gint
 gnome_print_pdf_page_free (GnomePrintPdfPage *page)
 {
@@ -3158,7 +3073,7 @@ gnome_print_pdf_free_fonts (GnomePrintPdf *pdf)
 		g_free (font->font_name);
 		for (i = 0; i < font->glyphs_num; i++) {
 			gchar *name;
-			name = font->glyphs [i];
+			name = font->glyphs[i].name;
 			g_free (name);
 		}
 		g_free (font->glyphs);
@@ -3235,13 +3150,12 @@ gnome_print_pdf_finalize (GtkObject *object)
 
 	gnome_print_pdf_free_pages (pdf, &showpaged_all);
 
-	if (!showpaged_all)
+	if (!showpaged_all && (g_list_length(pdf->pages)>0))
 		g_warning ("The application didn't called \"showpage\" for\n"
 							 "one or more pages. Please report this bug for the\n"
 							 "program you are using to print. Some **CRITICAL**\n"
 							 "messages are normal because of this bug.\n"
-							 "THIS IS NOT A GNOME-PRINT BUG !!\n\n"
-							 "Love all, serve all\n");
+							 "This is not a gnome-print bug.\n\n");
 
 	g_list_free (pdf->objects);
 	g_list_free (pdf->pages);
@@ -3346,4 +3260,3 @@ gnome_print_pdf_class_init (GnomePrintPdfClass *class)
 
   pc_class->close           = gnome_print_pdf_close;
 }
-
