@@ -561,7 +561,8 @@ gnome_print_ps_grayimage (GnomePrintContext *pc, const char *data, int width, in
 }
 
 static int
-gnome_print_ps_rgbimage (GnomePrintContext *pc, const char *data, int width, int height, int rowstride)
+gnome_print_ps_rgbimage (GnomePrintContext *pc, const char *data, int width, int height,
+					int rowstride)
 {
   return gnome_print_ps_image (pc, data, width, height, rowstride, 3);
 }
