@@ -40,6 +40,7 @@
 
 #include <libart_lgpl/art_affine.h>
 #include <libart_lgpl/art_misc.h>
+#include <libgnome/gnome-paper.h>
 #include <libgnomeprint/gp-unicode.h>
 #include <libgnomeprint/gnome-print-private.h>
 #include <libgnomeprint/gnome-printer-private.h>
@@ -217,6 +218,17 @@ gnome_print_ps2_get_date (void)
 	return date;
 }
 
+static gchar *
+gnome_print_ps2_print_ctm (const double *ctm)
+{
+	return g_strdup_printf ("- CTM.[%g,%g,%g,%g]",
+													ctm [0],
+													ctm [1],
+													ctm [2],
+													ctm [3]);
+													
+}
+
 GnomePrintPs2 *
 gnome_print_ps2_new (GnomePrinter *printer)
 {
@@ -225,6 +237,8 @@ gnome_print_ps2_new (GnomePrinter *printer)
 	gchar *date;
 	gint ret = 0;
 
+	g_return_val_if_fail (printer != NULL, NULL);
+	
 	ps2 = gtk_type_new (gnome_print_ps2_get_type ());
 
 	if (!gnome_print_context_open_file (GNOME_PRINT_CONTEXT (ps2), printer->filename))
@@ -511,14 +525,10 @@ gnome_print_ps2_showpage (GnomePrintContext *pc)
 
 	gnome_print_ps2_page_end   (pc);
 
-	/* We need to clean the Graphic State. Chema */
-#if 0	/* I don't see the need for cleaning the state, this is actually
-				 wrong, since postscript does not function that way */
 	gnome_print_ps2_graphic_state_free (ps2->graphic_state);
 	gnome_print_ps2_graphic_state_free (ps2->graphic_state_set);
 	ps2->graphic_state     = gnome_print_ps2_graphic_state_new (FALSE);
 	ps2->graphic_state_set = gnome_print_ps2_graphic_state_new (TRUE);
-#endif	
 	
 	gnome_print_ps2_page_start (pc);
 
@@ -633,7 +643,7 @@ gnome_print_ps2_setrgbcolor (GnomePrintContext *pc,
 }
 
 static gint
-gnome_print_ps2_setlinewidth (GnomePrintContext *pc, double linewidth)
+gnome_print_ps2_setlinewidth (GnomePrintContext *pc, double width)
 {
 	GnomePrintPs2 *ps2;
 	GnomePrintPs2GraphicState *gs;
@@ -644,12 +654,22 @@ gnome_print_ps2_setlinewidth (GnomePrintContext *pc, double linewidth)
 	ps2 = GNOME_PRINT_PS2 (pc);
 	g_return_val_if_fail (ps2 != NULL, -1);
 
-	if (linewidth < 0)
-		return gnome_print_ps2_error (FALSE, "setlinewidth, invalid parameter range %g", linewidth);
+	if (width < 0)
+		return gnome_print_ps2_error (FALSE, "setlinewidth, invalid parameter range %g", width);
 	
 	gs = gnome_print_ps2_graphic_state_current (ps2, TRUE);
 
+#if 0	
 	gs->linewidth = ((gs->ctm[0] + gs->ctm[3]) / 2) * linewidth;
+#else
+/* This is an ugly hack, but it should work ok for 90 deg.
+	 angles */
+	gs->linewidth = (fabs (width * gs->ctm[0]) +
+									 fabs (width * gs->ctm[1]) +
+									 fabs (width * gs->ctm[2]) +
+									 fabs (width * gs->ctm[3])) / 2;
+	
+#endif	
 
 	return 0;
 }
@@ -1269,6 +1289,14 @@ gnome_print_ps2_graphic_state_duplicate (GnomePrintPs2GraphicState *gs_in, Gnome
 
 	gs_out->current_path = gp_path_duplicate (gs_in->current_path);
 
+#if 0	
+	g_print ("Duplicate, CTM_in :%i [%s] CTM_out:%i %s\n",
+					 GPOINTER_TO_INT (gs_in->ctm),
+					 gnome_print_ps2_print_ctm (gs_in->ctm),
+					 GPOINTER_TO_INT (gs_out->ctm),
+					 gnome_print_ps2_print_ctm (gs_out->ctm));
+#endif	
+
 	return gs_out;
 }
 
@@ -1297,6 +1325,11 @@ gnome_print_ps2_gsave (GnomePrintContext *pc)
 
 	ps2->gsave_level_number++;
 
+	if (ps2->gsave_level_number == ps2->gsave_level_max) {
+		ps2->gsave_stack = g_realloc (ps2->gsave_stack, sizeof (GnomePrintPs2Gsave) *
+																	(ps2->gsave_level_max += GNOME_PRINT_PS2_NUMBER_OF_ELEMENTS_GROW));
+	}
+			
 	ret += gnome_print_context_fprintf (pc, "q" EOL);
 
 	return ret;
@@ -1365,8 +1398,10 @@ gnome_print_ps2_show_matrix_set (GnomePrintPs2 *ps2)
 	gs     = ps2->graphic_state;
 	gs_set = ps2->graphic_state_set;
 
+#if 0	/* I am not sure about this ... */
 	if (gnome_print_ps2_ctm_is_identity (gs->ctm))
 		return 0;
+#endif	
 	
 	ret += gnome_print_ps2_gsave (pc);
 	ret += gnome_print_context_fprintf (pc,
@@ -2108,7 +2143,16 @@ gnome_print_ps2_concat (GnomePrintContext *pc, const double matrix[6])
 
 	gs = gnome_print_ps2_graphic_state_current (ps2, FALSE);
 
+#if 0
+	g_print ("Concat %s with %s\n",
+					 gnome_print_ps2_print_ctm (gs->ctm),
+					 gnome_print_ps2_print_ctm (matrix));
+#endif	
 	art_affine_multiply (gs->ctm, matrix, gs->ctm);
+#if 0	
+	g_print ("AfterConcat %s\n",
+					 gnome_print_ps2_print_ctm (gs->ctm));
+#endif	
 	gnome_print_ps2_matrix_clean (gs->ctm);
 
 	return 0;
@@ -2438,7 +2482,7 @@ gnome_print_ps2_destroy (GtkObject *object)
 		showpaged_all = FALSE;
 	else
 		showpaged_all = TRUE;
-	
+
 	/* Free Pages */
 	for (list = ps2->pages; list != NULL; list = list->next) {
 		page = (GnomePrintPs2Page *) list->data;
@@ -2449,13 +2493,12 @@ gnome_print_ps2_destroy (GtkObject *object)
 	gnome_print_ps2_page_free (ps2->current_page);
 	g_list_free (ps2->pages);
 
-	if (!showpaged_all)
+	if (!showpaged_all && (g_list_length(ps2->pages)>0))
 		g_warning ("The application didn't called \"showpage\" for\n"
 							 "one or more pages. Please report this bug for the\n"
 							 "program you are using to print. Some **CRITICAL**\n"
 							 "messages are normal because of this bug.\n"
-							 "THIS IS NOT A GNOME-PRINT BUG !!\n\n"
-							 "Love all, serve all\n");
+							 "This is not a gnome-print bug.\n\n");
 
 
 	for (n=0; n < ps2->fonts_number; n++) {
@@ -2511,7 +2554,7 @@ gnome_print_ps2_init (GnomePrintPs2 *ps2)
 	ps2->gsave_level_max = GNOME_PRINT_PS2_NUMBER_OF_ELEMENTS;
 	ps2->gsave_level_number = 0;
 	ps2->gsave_stack = g_new (GnomePrintPs2Gsave, ps2->gsave_level_max);
-	
+
 }
 
 static void
