@@ -40,6 +40,7 @@
 #include "art_rgba_svp.h"
 #include "art_rgba_rgba_affine.h"
 
+#include <libgnomeprint/gp-unicode.h>
 #include <libgnomeprint/gnome-print-private.h>
 #include <libgnomeprint/gnome-print-rbuf.h>
 #include <libgnomeprint/gnome-print-rbuf-private.h>
@@ -57,9 +58,6 @@ struct _GnomePrintRBufPrivate {
 	gint rowstride;
 
 	gdouble page2buf[6];
-
-	GPGC * gc;
-
 	guint32 alpha : 1;
 };
 
@@ -67,42 +65,15 @@ static void gpb_class_init (GnomePrintRBufClass *class);
 static void gpb_init (GnomePrintRBuf *rbuf);
 static void gpb_destroy (GtkObject *object);
 
-static gint gpb_fill (GnomePrintContext * pc);
-static gint gpb_eofill (GnomePrintContext * pc);
+static gint gpb_fill (GnomePrintContext * pc, ArtWindRule rule);
 static gint gpb_stroke (GnomePrintContext * pc);
-static gint gpb_show_ucs4 (GnomePrintContext * pc, guint32 * buf, gint length);
+static gint gpb_show_sized (GnomePrintContext * pc, const char *text, int bytes);
 
 static gint gpb_grayimage (GnomePrintContext * pc, const gchar * data, gint width, gint height, gint rowstride);
 static gint gpb_rgbimage (GnomePrintContext * pc, const gchar * data, gint width, gint height, gint rowstride);
 static gint gpb_rgbaimage (GnomePrintContext * pc, const gchar * data, gint width, gint height, gint rowstride);
 
-static gint gpb_gsave (GnomePrintContext * pc);
-static gint gpb_grestore (GnomePrintContext * pc);
-
-static gint gpb_newpath (GnomePrintContext * pc);
-static gint gpb_moveto (GnomePrintContext * pc, gdouble x, gdouble y);
-static gint gpb_lineto (GnomePrintContext * pc, gdouble x, gdouble y);
-static gint gpb_curveto (GnomePrintContext * pc, gdouble x1, gdouble y1, gdouble x2, gdouble y2, gdouble x3, gdouble y3);
-static gint gpb_closepath (GnomePrintContext * pc);
-static gint gpb_strokepath (GnomePrintContext * pc);
-
-static gint gpb_setrgbcolor (GnomePrintContext * pc, gdouble r, gdouble g, gdouble b);
-static gint gpb_setlinewidth (GnomePrintContext * pc, gdouble width);
-static gint gpb_setmiterlimit (GnomePrintContext * pc, gdouble limit);
-static gint gpb_setlinejoin (GnomePrintContext * pc, gint jointype);
-static gint gpb_setlinecap (GnomePrintContext * pc, gint captype);
-static gint gpb_setdash (GnomePrintContext * pc, gint n_values, const gdouble * values, gdouble offset);
-static gint gpb_setopacity (GnomePrintContext * pc, gdouble opacity);
-
-static gint gpb_setfont (GnomePrintContext * pc, GnomeFont * font);
-
-static gint gpb_concat (GnomePrintContext * pc, const gdouble matrix[6]);
-#ifdef SETMATRIX_DEPRECATED
-static gint gpb_setmatrix (GnomePrintContext * pc, const gdouble matrix[6]);
-#endif 
-
-static gint gpb_clip (GnomePrintContext * pc);
-static gint gpb_eoclip (GnomePrintContext * pc);
+static gint gpb_clip (GnomePrintContext * pc, ArtWindRule rule);
 
 static gint gpb_textline (GnomePrintContext * pc, GnomeTextLine * textline);
 
@@ -110,10 +81,10 @@ static gint gpb_beginpage (GnomePrintContext * pc, const gchar * name);
 static gint gpb_showpage (GnomePrintContext * pc);
 static gint gpb_close (GnomePrintContext * pc);
 
-static void gp_svp_uncross_to_render (GnomePrintRBufPrivate * rbp, const ArtSVP * svp, ArtWindRule rule);
-static void gp_vpath_to_render (GnomePrintRBufPrivate * rbp, const ArtBpath * bpath, ArtWindRule rule);
+static void gp_svp_uncross_to_render (GnomePrintContext * pc, const ArtSVP * svp, ArtWindRule rule);
+static void gp_vpath_to_render (GnomePrintContext * pc, const ArtBpath * bpath, ArtWindRule rule);
 
-static void gp_render_silly_rgba (GnomePrintRBufPrivate * rbp, const guchar * pixels, gint width, gint height, gint rowstride);
+static void gp_render_silly_rgba (GnomePrintContext * pc, const guchar * pixels, gint width, gint height, gint rowstride);
 
 static GnomePrintContextClass * print_rbuf_parent_class;
 
@@ -161,39 +132,13 @@ gpb_class_init (GnomePrintRBufClass *class)
 	object_class->destroy = gpb_destroy;
 	
 	pc_class->fill = gpb_fill;
-	pc_class->eofill = gpb_eofill;
 	pc_class->stroke = gpb_stroke;
-	pc_class->show_ucs4 = gpb_show_ucs4;
+	pc_class->show_sized = gpb_show_sized;
 	pc_class->grayimage = gpb_grayimage;
 	pc_class->rgbimage = gpb_rgbimage;
 	pc_class->rgbaimage = gpb_rgbaimage;
 
-	pc_class->newpath = gpb_newpath;
-	pc_class->moveto = gpb_moveto;
-	pc_class->lineto = gpb_lineto;
-	pc_class->curveto = gpb_curveto;
-	pc_class->closepath = gpb_closepath;
-	pc_class->strokepath = gpb_strokepath;
-
-	pc_class->setrgbcolor = gpb_setrgbcolor;
-	pc_class->setopacity = gpb_setopacity;
-	pc_class->setlinewidth = gpb_setlinewidth;
-	pc_class->setmiterlimit = gpb_setmiterlimit;
-	pc_class->setlinejoin = gpb_setlinejoin;
-	pc_class->setlinecap = gpb_setlinecap;
-	pc_class->setdash = gpb_setdash;
-
-	pc_class->setfont = gpb_setfont;
-
-	pc_class->concat = gpb_concat;
-#ifdef SETMATRIX_DEPRECATED
-	pc_class->setmatrix = gpb_setmatrix;
-#endif	
-	pc_class->gsave = gpb_gsave;
-	pc_class->grestore = gpb_grestore;
-
 	pc_class->clip = gpb_clip;
-	pc_class->eoclip = gpb_eoclip;
 
 	pc_class->textline = gpb_textline;
 	pc_class->showpage = gpb_showpage;
@@ -211,7 +156,9 @@ gpb_init (GnomePrintRBuf * rbuf)
 
 	rbuf->private->pixels = NULL;
 	art_affine_identity (rbuf->private->page2buf);
+#if 0
 	rbuf->private->gc = NULL;
+#endif
 }
 
 static void
@@ -222,9 +169,6 @@ gpb_destroy (GtkObject *object)
 	rbuf = GNOME_PRINT_RBUF (object);
 
 	if (rbuf->private) {
-		if (rbuf->private->gc) {
-			gp_gc_unref (rbuf->private->gc);
-		}
 		g_free (rbuf->private);
 	}
 
@@ -237,13 +181,11 @@ gpb_destroy (GtkObject *object)
  */
 
 static gint
-gpb_fill (GnomePrintContext * pc)
+gpb_fill (GnomePrintContext * pc, ArtWindRule rule)
 {
 	GnomePrintRBuf * rbuf;
 	GnomePrintRBufPrivate * rbp;
-	GPGC * gc;
 	const GPPath * gppath;
-	GPPath * closedpath;
 	ArtBpath * bpath;
 
 	g_return_val_if_fail (pc != NULL, -1);
@@ -251,61 +193,16 @@ gpb_fill (GnomePrintContext * pc)
 
 	rbuf = GNOME_PRINT_RBUF (pc);
 	rbp = rbuf->private;
-	gc = rbuf->private->gc;
 
-	g_return_val_if_fail (gp_gc_has_currentpath (gc), -1);
+	g_return_val_if_fail (gp_gc_has_currentpath (pc->gc), -1);
 
-	gppath = gp_gc_get_currentpath (gc);
+	gppath = gp_gc_get_currentpath (pc->gc);
 	g_assert (gppath != NULL);
 
-	closedpath = gp_path_close_all (gppath);
-	g_assert (closedpath != NULL);
-
-	bpath = gp_path_bpath (closedpath);
+	bpath = gp_path_bpath (gppath);
 	g_assert (bpath != NULL);
 
-	gp_vpath_to_render (rbp, bpath, ART_WIND_RULE_NONZERO);
-
-	gp_path_unref (closedpath);
-
-	gp_gc_newpath (gc);
-
-	return 1;
-}
-
-static gint
-gpb_eofill (GnomePrintContext * pc)
-{
-	GnomePrintRBuf * rbuf;
-	GnomePrintRBufPrivate * rbp;
-	GPGC * gc;
-	const GPPath * gppath;
-	GPPath * closedpath;
-	ArtBpath * bpath;
-
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_RBUF (pc), -1);
-
-	rbuf = GNOME_PRINT_RBUF (pc);
-	rbp = rbuf->private;
-	gc = rbuf->private->gc;
-
-	g_return_val_if_fail (gp_gc_has_currentpath (gc), -1);
-
-	gppath = gp_gc_get_currentpath (gc);
-	g_assert (gppath != NULL);
-
-	closedpath = gp_path_close_all (gppath);
-	g_assert (closedpath != NULL);
-
-	bpath = gp_path_bpath (closedpath);
-	g_assert (bpath != NULL);
-
-	gp_vpath_to_render (rbp, bpath, ART_WIND_RULE_ODDEVEN);
-
-	gp_path_unref (closedpath);
-
-	gp_gc_newpath (gc);
+	gp_vpath_to_render (pc, bpath, rule);
 
 	return 1;
 }
@@ -317,10 +214,9 @@ gpb_stroke (GnomePrintContext * pc)
 {
 	GnomePrintRBuf * rbuf;
 	GnomePrintRBufPrivate * rbp;
-	GPGC * gc;
 	const GPPath * gppath;
 	ArtBpath * bpath;
-	ArtVpath * vpath;
+	ArtVpath * vpath, * pvpath;
 	ArtSVP * svp;
 	const ArtVpathDash * dash;
 	gdouble linewidth;
@@ -330,11 +226,10 @@ gpb_stroke (GnomePrintContext * pc)
 
 	rbuf = GNOME_PRINT_RBUF (pc);
 	rbp = rbuf->private;
-	gc = rbuf->private->gc;
 
-	g_return_val_if_fail (gp_gc_currentpath_points (gc) > 1, -1);
+	g_return_val_if_fail (gp_gc_currentpath_points (pc->gc) > 1, -1);
 
-	gppath = gp_gc_get_currentpath (gc);
+	gppath = gp_gc_get_currentpath (pc->gc);
 	g_assert (gppath != NULL);
 
 	bpath = gp_path_bpath (gppath);
@@ -343,66 +238,57 @@ gpb_stroke (GnomePrintContext * pc)
 	vpath = art_bez_path_to_vec (bpath, 0.25);
 	g_assert (vpath != NULL);
 
-	dash = gp_gc_get_dash (gc);
+	pvpath = art_vpath_perturb (vpath);
+	art_free (vpath);
+
+	dash = gp_gc_get_dash (pc->gc);
 
 	if ((dash->n_dash > 0) && (dash->dash != NULL)) {
 		ArtVpath * dvp;
-		dvp = art_vpath_dash (vpath, dash);
+		dvp = art_vpath_dash (pvpath, dash);
 		g_assert (dvp != NULL);
-		art_free (vpath);
-		vpath = dvp;
+		art_free (pvpath);
+		pvpath = dvp;
 	}
 
 	/* fixme */
 
-	linewidth = gp_gc_get_linewidth (gc);
+	linewidth = gp_gc_get_linewidth (pc->gc);
 
-	svp = art_svp_vpath_stroke (vpath,
-		gp_gc_get_linejoin (gc),
-		gp_gc_get_linecap (gc),
+	svp = art_svp_vpath_stroke (pvpath,
+		gp_gc_get_linejoin (pc->gc),
+		gp_gc_get_linecap (pc->gc),
 		linewidth,
-		gp_gc_get_miterlimit (gc),
+		gp_gc_get_miterlimit (pc->gc),
 		0.25);
 	g_assert (svp != NULL);
-	art_free (vpath);
+	art_free (pvpath);
 
-	gp_svp_uncross_to_render (rbp, svp, ART_WIND_RULE_NONZERO);
+	gp_svp_uncross_to_render (pc, svp, ART_WIND_RULE_NONZERO);
 
 	art_svp_free (svp);
 
-	/* fixme: is this correct? */
-	gp_gc_newpath (gc);
-
 	return 1;
 }
 
 /* fixme: */
 
 static gint
-gpb_show_ucs4 (GnomePrintContext * pc, guint32 * buf, gint length)
+gpb_show_sized (GnomePrintContext * pc, const char *text, int bytes)
 {
 	GnomePrintRBuf * rbuf;
 	GnomePrintRBufPrivate * rbp;
-	GPGC * gc;
 	const GnomeFont * font;
-#if 0
-	GnomeFontMap * fm;
-#else
 	const GnomeFontFace * face;
 	gdouble size;
 	const ArtBpath * bpath;
 	ArtPoint advance;
 	gint code;
-#endif
 	const ArtPoint * p;
-#if 0
-	Gt1LoadedFont * gt1f;
-	Gt1GlyphOutline * gol;
-#endif
 	gdouble affine[6];
 	const gdouble * ctm;
 	gdouble x, y;
-	gint i;
+	const char *tptr;
 	ArtBpath * abp;
 
 	g_return_val_if_fail (pc != NULL, 0);
@@ -410,29 +296,20 @@ gpb_show_ucs4 (GnomePrintContext * pc, guint32 * buf, gint length)
 
 	rbuf = GNOME_PRINT_RBUF (pc);
 	rbp = rbuf->private;
-	gc = rbuf->private->gc;
 
-	font = gp_gc_get_font (gc);
-#if 0
-	fm = font->fontmap_entry;
-	g_return_val_if_fail (fm != NULL, -1);
-
-	gt1f = gt1_load_font (fm->pfb_fn);
-	g_assert (gt1f != NULL);
-#else
+	font = gp_gc_get_font (pc->gc);
 	face = gnome_font_get_face ((GnomeFont *) font);
 	size = gnome_font_get_size ((GnomeFont *) font);
-#endif
 
-	p = gp_gc_get_currentpoint (gc);
+	p = gp_gc_get_currentpoint (pc->gc);
 	g_return_val_if_fail (p != NULL, -1);
 
-	ctm = gp_gc_get_ctm (gc);
+	ctm = gp_gc_get_ctm (pc->gc);
 
 	x = y = 0.0;
 
-	for (i = 0; i < length; i++) {
-		code = gnome_font_face_lookup_default ((GnomeFontFace *) face, GUINT32_FROM_BE (buf[i]));
+	for (tptr = text; tptr && tptr < (text + bytes); tptr = g_utf8_next_char (tptr)) {
+		code = gnome_font_face_lookup_default ((GnomeFontFace *) face, g_utf8_get_char (tptr));
 		bpath = gnome_font_face_get_glyph_stdoutline ((GnomeFontFace *) face, code);
 
 		art_affine_scale (affine,
@@ -444,30 +321,14 @@ gpb_show_ucs4 (GnomePrintContext * pc, guint32 * buf, gint length)
 		affine[4] = affine[4] - ctm[4] + p->x;
 		affine[5] = affine[5] - ctm[5] + p->y;
 
-#if 0
-		abp = art_bpath_affine_transform (gol->bpath, affine);
-		g_assert (abp != NULL);
-#else
 		abp = art_bpath_affine_transform (bpath, affine);
-#endif
 
-		gp_vpath_to_render (rbp, abp, ART_WIND_RULE_NONZERO);
+		gp_vpath_to_render (pc, abp, ART_WIND_RULE_NONZERO);
 
-#if 0
-		x += gol->wx * font->size / 1000.0;
-#else
 		gnome_font_face_get_glyph_stdadvance ((GnomeFontFace *) face, code, &advance);
 		x += advance.x * size / 1000.0;
-#endif
 	}
 
-#if 0
-	gt1_unload_font (gt1f);
-#endif
-
-	/* fixme: is this correct? */
-	gp_gc_newpath (gc);
-
 	return 1;
 }
 
@@ -478,7 +339,6 @@ gpb_grayimage (GnomePrintContext * pc, const gchar * data, gint width, gint heig
 {
 	GnomePrintRBuf * rbuf;
 	GnomePrintRBufPrivate * rbp;
-	GPGC * gc;
 	art_u8 * ib, * ipd;
 	const art_u8 * ips;
 	gint x, y;
@@ -491,7 +351,6 @@ gpb_grayimage (GnomePrintContext * pc, const gchar * data, gint width, gint heig
 
 	rbuf = GNOME_PRINT_RBUF (pc);
 	rbp = rbuf->private;
-	gc = rbuf->private->gc;
 
 	ib = g_new (art_u8, width * height * 4);
 
@@ -506,7 +365,7 @@ gpb_grayimage (GnomePrintContext * pc, const gchar * data, gint width, gint heig
 		}
 	}
 
-	gp_render_silly_rgba (rbp, ib, width, height, rowstride);
+	gp_render_silly_rgba (pc, ib, width, height, rowstride);
 
 	g_free (ib);
 
@@ -518,7 +377,6 @@ gpb_rgbimage (GnomePrintContext * pc, const gchar * data, gint width, gint heigh
 {
 	GnomePrintRBuf * rbuf;
 	GnomePrintRBufPrivate * rbp;
-	GPGC * gc;
 	art_u8 * ib, * ipd;
 	const art_u8 * ips;
 	gint x, y;
@@ -531,7 +389,6 @@ gpb_rgbimage (GnomePrintContext * pc, const gchar * data, gint width, gint heigh
 
 	rbuf = GNOME_PRINT_RBUF (pc);
 	rbp = rbuf->private;
-	gc = rbuf->private->gc;
 
 	ib = g_new (art_u8, width * height * 4);
 
@@ -546,7 +403,7 @@ gpb_rgbimage (GnomePrintContext * pc, const gchar * data, gint width, gint heigh
 		}
 	}
 
-	gp_render_silly_rgba (rbp, ib, width, height, width * 4);
+	gp_render_silly_rgba (pc, ib, width, height, width * 4);
 
 	g_free (ib);
 
@@ -560,7 +417,6 @@ gpb_rgbaimage (GnomePrintContext * pc, const gchar * data, gint width, gint heig
 {
 	GnomePrintRBuf * rbuf;
 	GnomePrintRBufPrivate * rbp;
-	GPGC * gc;
 
 	g_return_val_if_fail (pc != NULL, 0);
 	g_return_val_if_fail (GNOME_IS_PRINT_RBUF (pc), 0);
@@ -570,347 +426,16 @@ gpb_rgbaimage (GnomePrintContext * pc, const gchar * data, gint width, gint heig
 
 	rbuf = GNOME_PRINT_RBUF (pc);
 	rbp = rbuf->private;
-	gc = rbuf->private->gc;
 
-	gp_render_silly_rgba (rbp, data, width, height, rowstride);
+	gp_render_silly_rgba (pc, data, width, height, rowstride);
 
 	return 1;
 }
 
-/*
- * Gsave & Grestore
- */
-
-static gint
-gpb_gsave (GnomePrintContext * pc)
-{
-	GnomePrintRBuf * rbuf;
-
-	g_return_val_if_fail (pc != NULL, 0);
-	g_return_val_if_fail (GNOME_IS_PRINT_RBUF (pc), 0);
-
-	rbuf = GNOME_PRINT_RBUF (pc);
-
-	gp_gc_gsave (rbuf->private->gc);
-
-	return 1;
-}
-
-static gint
-gpb_grestore (GnomePrintContext * pc)
-{
-	GnomePrintRBuf * rbuf;
-
-	g_return_val_if_fail (pc != NULL, 0);
-	g_return_val_if_fail (GNOME_IS_PRINT_RBUF (pc), 0);
-
-	rbuf = GNOME_PRINT_RBUF (pc);
-
-	gp_gc_grestore (rbuf->private->gc);
-
-	return 1;
-}
-
-
-/*
- * Path operations
- */
-
-static gint
-gpb_newpath (GnomePrintContext * pc)
-{
-	GnomePrintRBuf * rbuf;
-
-	g_return_val_if_fail (pc != NULL, 0);
-	g_return_val_if_fail (GNOME_IS_PRINT_RBUF (pc), 0);
-
-	rbuf = GNOME_PRINT_RBUF (pc);
-
-	gp_gc_newpath (rbuf->private->gc);
-
-	return 1;
-}
-
-static gint
-gpb_moveto (GnomePrintContext * pc, gdouble x, gdouble y)
-{
-	GnomePrintRBuf * rbuf;
-
-	g_return_val_if_fail (pc != NULL, 0);
-	g_return_val_if_fail (GNOME_IS_PRINT_RBUF (pc), 0);
-
-	rbuf = GNOME_PRINT_RBUF (pc);
-
-	gp_gc_moveto (rbuf->private->gc, x, y);
-
-	return 1;
-}
-
-static gint
-gpb_lineto (GnomePrintContext * pc, gdouble x, gdouble y)
-{
-	GnomePrintRBuf * rbuf;
-
-	g_return_val_if_fail (pc != NULL, 0);
-	g_return_val_if_fail (GNOME_IS_PRINT_RBUF (pc), 0);
-
-	rbuf = GNOME_PRINT_RBUF (pc);
-
-	gp_gc_lineto (rbuf->private->gc, x, y);
-
-	return 1;
-}
-
-static gint
-gpb_curveto (GnomePrintContext * pc, gdouble x1, gdouble y1, gdouble x2, gdouble y2, gdouble x3, gdouble y3)
-{
-	GnomePrintRBuf * rbuf;
-
-	g_return_val_if_fail (pc != NULL, 0);
-	g_return_val_if_fail (GNOME_IS_PRINT_RBUF (pc), 0);
-
-	rbuf = GNOME_PRINT_RBUF (pc);
-
-	gp_gc_curveto (rbuf->private->gc, x1, y1, x2, y2, x3, y3);
-
-	return 1;
-}
-
-static gint
-gpb_closepath (GnomePrintContext * pc)
-{
-	GnomePrintRBuf * rbuf;
-
-	g_return_val_if_fail (pc != NULL, 0);
-	g_return_val_if_fail (GNOME_IS_PRINT_RBUF (pc), 0);
-
-	rbuf = GNOME_PRINT_RBUF (pc);
-
-	gp_gc_closepath (rbuf->private->gc);
-
-	return 1;
-}
-
-static gint
-gpb_strokepath (GnomePrintContext * pc)
-{
-	GnomePrintRBuf * rbuf;
-
-	g_return_val_if_fail (pc != NULL, 0);
-	g_return_val_if_fail (GNOME_IS_PRINT_RBUF (pc), 0);
-
-	rbuf = GNOME_PRINT_RBUF (pc);
-
-	gp_gc_strokepath (rbuf->private->gc);
-
-	return 1;
-}
-
-/*
- * Simple GC Properties
- */
-
-static gint
-gpb_setrgbcolor (GnomePrintContext * pc, gdouble r, gdouble g, gdouble b)
-{
-	GnomePrintRBuf * rbuf;
-
-	g_return_val_if_fail (pc != NULL, 0);
-	g_return_val_if_fail (GNOME_IS_PRINT_RBUF (pc), 0);
-
-	rbuf = GNOME_PRINT_RBUF (pc);
-
-	gp_gc_set_rgbcolor (rbuf->private->gc, r, g, b);
-
-	return 1;
-}
-
-static gint
-gpb_setopacity (GnomePrintContext * pc, gdouble opacity)
-{
-	GnomePrintRBuf * rbuf;
-
-	g_return_val_if_fail (pc != NULL, 0);
-	g_return_val_if_fail (GNOME_IS_PRINT_RBUF (pc), 0);
-
-	rbuf = GNOME_PRINT_RBUF (pc);
-
-	gp_gc_set_opacity (rbuf->private->gc, opacity);
-
-	return 1;
-}
-
-static gint
-gpb_setlinewidth (GnomePrintContext * pc, gdouble width)
-{
-	GnomePrintRBuf * rbuf;
-
-	g_return_val_if_fail (pc != NULL, 0);
-	g_return_val_if_fail (GNOME_IS_PRINT_RBUF (pc), 0);
-
-	rbuf = GNOME_PRINT_RBUF (pc);
-
-	gp_gc_set_linewidth (rbuf->private->gc, width);
-
-	return 1;
-}
-
-static gint
-gpb_setmiterlimit (GnomePrintContext * pc, gdouble limit)
-{
-	GnomePrintRBuf * rbuf;
-
-	g_return_val_if_fail (pc != NULL, 0);
-	g_return_val_if_fail (GNOME_IS_PRINT_RBUF (pc), 0);
-
-	rbuf = GNOME_PRINT_RBUF (pc);
-
-	gp_gc_set_miterlimit (rbuf->private->gc, limit);
-
-	return 1;
-}
-
-static gint
-gpb_setlinejoin (GnomePrintContext * pc, gint jointype)
-{
-	GnomePrintRBuf * rbuf;
-
-	g_return_val_if_fail (pc != NULL, 0);
-	g_return_val_if_fail (GNOME_IS_PRINT_RBUF (pc), 0);
-
-	rbuf = GNOME_PRINT_RBUF (pc);
-
-	switch (jointype) {
-	case 0:
-		gp_gc_set_linejoin (rbuf->private->gc, ART_PATH_STROKE_JOIN_MITER);
-		break;
-	case 1:
-		gp_gc_set_linejoin (rbuf->private->gc, ART_PATH_STROKE_JOIN_ROUND);
-		break;
-	case 2:
-		gp_gc_set_linejoin (rbuf->private->gc, ART_PATH_STROKE_JOIN_BEVEL);
-		break;
-	default:
-		return -1;
-		break;
-	}
-
-	return 1;
-}
-
-static gint
-gpb_setlinecap (GnomePrintContext * pc, gint captype)
-{
-	GnomePrintRBuf * rbuf;
-
-	g_return_val_if_fail (pc != NULL, 0);
-	g_return_val_if_fail (GNOME_IS_PRINT_RBUF (pc), 0);
-
-	rbuf = GNOME_PRINT_RBUF (pc);
-
-	switch (captype) {
-	case 0:
-		gp_gc_set_linecap (rbuf->private->gc, ART_PATH_STROKE_CAP_BUTT);
-		break;
-	case 1:
-		gp_gc_set_linecap (rbuf->private->gc, ART_PATH_STROKE_CAP_ROUND);
-		break;
-	case 2:
-		gp_gc_set_linecap (rbuf->private->gc, ART_PATH_STROKE_CAP_SQUARE);
-		break;
-	default:
-		return -1;
-		break;
-	}
-
-	return 1;
-}
-
-static gint
-gpb_setdash (GnomePrintContext * pc, gint n_values, const gdouble * values, gdouble offset)
-{
-	GnomePrintRBuf * rbuf;
-
-	g_return_val_if_fail (pc != NULL, 0);
-	g_return_val_if_fail (GNOME_IS_PRINT_RBUF (pc), 0);
-
-	rbuf = GNOME_PRINT_RBUF (pc);
-
-	gp_gc_set_dash (rbuf->private->gc, n_values, values, offset);
-
-	return 1;
-}
-
-/*
- * Font loading
- */
-
-static gint
-gpb_setfont (GnomePrintContext * pc, GnomeFont * font)
-{
-	GnomePrintRBuf * rbuf;
-
-	g_return_val_if_fail (pc != NULL, 0);
-	g_return_val_if_fail (GNOME_IS_PRINT_RBUF (pc), 0);
-	g_return_val_if_fail (font != NULL, 0);
-	g_return_val_if_fail (GNOME_IS_FONT (font), 0);
-
-	rbuf = GNOME_PRINT_RBUF (pc);
-
-	gp_gc_set_font (rbuf->private->gc, font);
-
-	return 1;
-}
-
-/*
- * CTM transformation
- */
-
-static gint
-gpb_concat (GnomePrintContext * pc, const gdouble matrix[6])
-{
-	GnomePrintRBuf * rbuf;
-
-	g_return_val_if_fail (pc != NULL, 0);
-	g_return_val_if_fail (GNOME_IS_PRINT_RBUF (pc), 0);
-	g_return_val_if_fail (matrix != NULL, 0);
-
-	rbuf = GNOME_PRINT_RBUF (pc);
-
-	gp_gc_concat (rbuf->private->gc, matrix);
-
-	return 1;
-}
-
-#ifdef SETMATRIX_DEPRECATED
-static gint
-gpb_setmatrix (GnomePrintContext * pc, const gdouble matrix[6])
-{
-	GnomePrintRBuf * rbuf;
-	gdouble m[6];
-
-	g_warning ("setmatrix is deprecated!");
-
-	g_return_val_if_fail (pc != NULL, 0);
-	g_return_val_if_fail (GNOME_IS_PRINT_RBUF (pc), 0);
-	g_return_val_if_fail (matrix != NULL, 0);
-
-	rbuf = GNOME_PRINT_RBUF (pc);
-
-	/* We have to preserve page2buf transform :) */
-
-	art_affine_multiply (m, matrix, rbuf->private->page2buf);
-
-	gp_gc_setmatrix (rbuf->private->gc, m);
-
-	return 1;
-}
-#endif
-
 /* Clipping */
 
 static gint
-gpb_clip (GnomePrintContext * pc)
+gpb_clip (GnomePrintContext * pc, ArtWindRule rule)
 {
 	GnomePrintRBuf * rbuf;
 
@@ -919,22 +444,11 @@ gpb_clip (GnomePrintContext * pc)
 
 	rbuf = GNOME_PRINT_RBUF (pc);
 
-	gp_gc_clip (rbuf->private->gc);
-
-	return 1;
-}
-
-static gint
-gpb_eoclip (GnomePrintContext * pc)
-{
-	GnomePrintRBuf * rbuf;
-
-	g_return_val_if_fail (pc != NULL, 0);
-	g_return_val_if_fail (GNOME_IS_PRINT_RBUF (pc), 0);
-
-	rbuf = GNOME_PRINT_RBUF (pc);
-
-	gp_gc_eoclip (rbuf->private->gc);
+	if (rule == ART_WIND_RULE_NONZERO) {
+		gp_gc_clip (pc->gc);
+	} else {
+		gp_gc_eoclip (pc->gc);
+	}
 
 	return 1;
 }
@@ -951,8 +465,6 @@ gpb_showpage (GnomePrintContext * pc)
 
 	rbuf = GNOME_PRINT_RBUF (pc);
 
-	gp_gc_reset (rbuf->private->gc);
-
 	return 1;
 }
 
@@ -983,10 +495,6 @@ gpb_beginpage (GnomePrintContext * pc, const gchar * name)
 {
 	GnomePrintRBuf * rbuf;
 	GnomePrintRBufPrivate * rbp;
-#if 0
-	guint32 * p;
-	gint x, y;
-#endif
 
 	g_return_val_if_fail (pc != NULL, 0);
 	g_return_val_if_fail (GNOME_IS_PRINT_RBUF (pc), 0);
@@ -994,25 +502,7 @@ gpb_beginpage (GnomePrintContext * pc, const gchar * name)
 	rbuf = GNOME_PRINT_RBUF (pc);
 	rbp = rbuf->private;
 
-	/* fixme: What exactly should be reset? */
-	gp_gc_reset (rbuf->private->gc);
-	gp_gc_concat (rbuf->private->gc, rbuf->private->page2buf);
-
-#if 0
-	if (rbp->alpha) {
-		for (y = 0; y < rbp->height; y++) {
-			p = (guint32 *) (rbp->pixels + y * rbp->rowstride);
-			for (x = 0; x < rbp->width; x++)
-				*p++ = 0xffffff00;
-	}
-	} else {
-		for (y = 0; y < rbp->height; y++) {
-			memset (rbp->pixels + y * rbp->rowstride,
-				0xff,
-				 rbp->width * 3);
-		}
-	}
-#endif
+	gnome_print_concat (pc, rbuf->private->page2buf);
 
 	return 1;
 }
@@ -1054,8 +544,6 @@ gnome_print_rbuf_construct (GnomePrintRBuf * rbuf,
 	gdouble page2buf[6],
 	gboolean alpha)
 {
-	GPGC * gc;
-
 	g_return_val_if_fail (rbuf != NULL, NULL);
 	g_return_val_if_fail (GNOME_IS_PRINT_RBUF (rbuf), NULL);
 	g_return_val_if_fail (pixels != NULL, NULL);
@@ -1074,25 +562,22 @@ gnome_print_rbuf_construct (GnomePrintRBuf * rbuf,
 
 	memcpy (rbuf->private->page2buf, page2buf, sizeof (gdouble) * 6);
 
-	gc = gp_gc_new ();
-	g_assert (gc != NULL);
-	gp_gc_concat (gc, page2buf);
-
-	rbuf->private->gc = gc;
-
 	return rbuf;
 }
 
 /* Private helpers */
 
 static void
-gp_svp_uncross_to_render (GnomePrintRBufPrivate * rbp, const ArtSVP * svp, ArtWindRule rule)
+gp_svp_uncross_to_render (GnomePrintContext * pc, const ArtSVP * svp, ArtWindRule rule)
 {
+	GnomePrintRBufPrivate * rbp;
 	ArtSVP * svp1, * svp2;
 
-	g_assert (rbp != NULL);
+	g_assert (pc != NULL);
 	g_assert (svp != NULL);
 
+	rbp = GNOME_PRINT_RBUF (pc)->private;
+
 	svp2 = art_svp_uncross ((ArtSVP *) svp);
 	g_assert (svp2 != NULL);
 
@@ -1100,8 +585,8 @@ gp_svp_uncross_to_render (GnomePrintRBufPrivate * rbp, const ArtSVP * svp, ArtWi
 	g_assert (svp1 != NULL);
 	art_svp_free (svp2);
 
-	if (gp_gc_has_clipsvp (rbp->gc)) {
-		svp2 = art_svp_intersect (svp1, gp_gc_get_clipsvp (rbp->gc));
+	if (gp_gc_has_clipsvp (pc->gc)) {
+		svp2 = art_svp_intersect (svp1, gp_gc_get_clipsvp (pc->gc));
 		g_assert (svp2 != NULL);
 		art_svp_free (svp1);
 		svp1 = svp2;
@@ -1110,13 +595,13 @@ gp_svp_uncross_to_render (GnomePrintRBufPrivate * rbp, const ArtSVP * svp, ArtWi
 	if (rbp->alpha) {
 		art_rgba_svp_alpha (svp1,
 			0, 0, rbp->width, rbp->height,
-			gp_gc_get_rgba (rbp->gc),
+			gp_gc_get_rgba (pc->gc),
 			rbp->pixels, rbp->rowstride,
 			NULL);
 	} else {
 		art_rgb_svp_alpha (svp1,
 			0, 0, rbp->width, rbp->height,
-			gp_gc_get_rgba (rbp->gc),
+			gp_gc_get_rgba (pc->gc),
 			rbp->pixels, rbp->rowstride,
 			NULL);
 	}
@@ -1125,14 +610,17 @@ gp_svp_uncross_to_render (GnomePrintRBufPrivate * rbp, const ArtSVP * svp, ArtWi
 }
 
 static void
-gp_vpath_to_render (GnomePrintRBufPrivate * rbp, const ArtBpath * bpath, ArtWindRule rule)
+gp_vpath_to_render (GnomePrintContext * pc, const ArtBpath * bpath, ArtWindRule rule)
 {
+	GnomePrintRBufPrivate * rbp;
 	ArtVpath * vpath1, * vpath2;
 	ArtSVP * svp;
 
-	g_assert (rbp != NULL);
+	g_assert (pc != NULL);
 	g_assert (bpath != NULL);
 
+	rbp = GNOME_PRINT_RBUF (pc)->private;
+
 	vpath1 = art_bez_path_to_vec (bpath, 0.25);
 	g_assert (vpath1 != NULL);
 
@@ -1144,18 +632,19 @@ gp_vpath_to_render (GnomePrintRBufPrivate * rbp, const ArtBpath * bpath, ArtWind
 	g_assert (svp != NULL);
 	art_free (vpath2);
 
-	gp_svp_uncross_to_render (rbp, svp, rule);
+	gp_svp_uncross_to_render (pc, svp, rule);
 
 	art_svp_free (svp);
 }
 
 static void
-gp_render_silly_rgba (GnomePrintRBufPrivate * rbp,
+gp_render_silly_rgba (GnomePrintContext * pc,
 	const guchar * pixels, gint width, gint height, gint rowstride)
 {
+	GnomePrintRBufPrivate * rbp;
 	gdouble affine[6];
 	ArtVpath vp[6];
-	ArtVpath * vpath;
+	ArtVpath * vpath, * pvpath;
 	ArtSVP * svp1, * svp2;
 	ArtDRect bbox, pbox;
 	ArtIRect ibox;
@@ -1164,9 +653,11 @@ gp_render_silly_rgba (GnomePrintRBufPrivate * rbp,
 	guchar * p, * ip, * cp;
 	gint bw, bh, x, y;
 
+	rbp = GNOME_PRINT_RBUF (pc)->private;
+
 	art_affine_scale (affine, 1.0 / width, -1.0 / height);
 	affine[5] = 1.0;
-	art_affine_multiply (affine, affine, gp_gc_get_ctm (rbp->gc));
+	art_affine_multiply (affine, affine, gp_gc_get_ctm (pc->gc));
 
 	vp[0].code = ART_MOVETO;
 	vp[0].x = 0.0;
@@ -1187,17 +678,20 @@ gp_render_silly_rgba (GnomePrintRBufPrivate * rbp,
 
 	vpath = art_vpath_affine_transform (vp, affine);
 
-	svp1 = art_svp_from_vpath (vpath);
+	pvpath = art_vpath_perturb (vpath);
 	art_free (vpath);
 
+	svp1 = art_svp_from_vpath (pvpath);
+	art_free (pvpath);
+
 	svp2 = art_svp_uncross (svp1);
 	art_svp_free (svp1);
 
 	svp1 = art_svp_rewind_uncrossed (svp2, ART_WIND_RULE_NONZERO);
 	art_svp_free (svp2);
 
-	if (gp_gc_has_clipsvp (rbp->gc)) {
-		svp2 = art_svp_intersect (svp1, gp_gc_get_clipsvp (rbp->gc));
+	if (gp_gc_has_clipsvp (pc->gc)) {
+		svp2 = art_svp_intersect (svp1, gp_gc_get_clipsvp (pc->gc));
 		art_svp_free (svp1);
 		svp1 = svp2;
 	}
