@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * gnome-print-rgbp.c: A preview driver abstract class for GnomePrint
  * that renders into an RGB buffer.
@@ -31,6 +32,9 @@ struct _GnomePrintRGBPPrivate {
 	unsigned int width;
 	unsigned int height;
 
+	/* Paper Name */
+	GnomePaper * paper;
+
 	int dpi;
 	GnomeCanvasBuf buf;
 };
@@ -72,7 +76,9 @@ gnome_print_rgbp_construct (GnomePrintRGBP *rgbp,
 		GNOME_PRINT_PREVIEW (rgbp),
 		GNOME_CANVAS (rgbp->priv->canvas),
 		paper_info);
-
+	
+	/* Save the paper name for multiple pages */
+	rgbp->priv->paper = (GnomePaper *) paper_info;
 
 	/* White background */
 	rgbp->priv->buf.bg_color = 0xffffff;
@@ -122,6 +128,7 @@ rgbp_finalize (GtkObject *object)
 
 	if (rgbp->priv->canvas)
 		gtk_object_unref (GTK_OBJECT (rgbp->priv->canvas));
+
 	g_free (rgbp->priv);
 	
 	GTK_OBJECT_CLASS (print_rgbp_parent_class)->finalize (object);
@@ -204,6 +211,19 @@ rgbp_showpage (GnomePrintContext *pc)
 
 		rgb_class->print_band (rgbp, rgbp->priv->buf.buf, &rgbp->priv->buf.rect);
 	}
+
+	/* We destroy the Canvas, and create a new one,
+	   so that we clear all the objects */
+	gtk_object_unref (GTK_OBJECT (rgbp->priv->canvas));
+	rgbp->priv->canvas = gnome_canvas_new_aa ();
+	gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (rgbp->priv->canvas), dpi/72);
+	gnome_print_preview_construct (
+		GNOME_PRINT_PREVIEW (rgbp),
+		GNOME_CANVAS (rgbp->priv->canvas),
+		rgbp->priv->paper);
+
+	rgb_class->page_end (rgbp);
+	
 	return 1;
 }
 
