@@ -118,8 +118,12 @@ gnome_print_context_new_with_paper_size (GnomePrinter *printer, const char *pape
 	/* This is just TEMPORARY !*/
 
 	if (strcmp (printer->driver, "gnome-print-ps") == 0) {
-		GnomePrintPs *ps = gnome_print_ps_new (printer);
-		GnomePrintContext *frgba = gnome_print_frgba_new ((GnomePrintContext *) ps);
+		GnomePrintPs *ps;
+		GnomePrintContext *frgba;
+		ps = gnome_print_ps_new (printer);
+		if (!GNOME_IS_PRINT_PS (ps))
+			return NULL;
+		frgba = gnome_print_frgba_new ((GnomePrintContext *) ps);
 		gtk_object_unref (GTK_OBJECT (ps));
 		return frgba ? frgba : NULL;
 	}
@@ -128,8 +132,12 @@ gnome_print_context_new_with_paper_size (GnomePrinter *printer, const char *pape
 		return ps ? GNOME_PRINT_CONTEXT (ps) : NULL;
 	}
 	if (strcmp (printer->driver, "gnome-print-ps2") == 0) {
-		GnomePrintPs2 *ps2 = gnome_print_ps2_new (printer);
-		GnomePrintContext *frgba = gnome_print_frgba_new ((GnomePrintContext *) ps2);
+		GnomePrintPs2 *ps2;
+		GnomePrintContext *frgba;
+		ps2 = gnome_print_ps2_new (printer);
+		if (!GNOME_IS_PRINT_PS2 (ps2))
+			return NULL;
+		frgba = gnome_print_frgba_new ((GnomePrintContext *) ps2);
 		gtk_object_unref (GTK_OBJECT (ps2));
 		return frgba ? frgba : NULL;
 	}
@@ -485,6 +493,9 @@ gnome_print_show_sized (GnomePrintContext *pc, char const *text, int bytes)
 
 	if (bytes < 1) return GNOME_PRINT_OK;
 
+	if (!g_utf8_validate (text, bytes, &invalid))
+		g_warning ("Could not UTF8 Validate *%s*", text);
+	
 	g_return_val_if_fail (g_utf8_validate (text, bytes, &invalid), GNOME_PRINT_ERROR_TEXTCORRUPT);
 
 	if (PRINT_CLASS (pc)->show_sized)
@@ -497,9 +508,6 @@ gnome_print_show_sized (GnomePrintContext *pc, char const *text, int bytes)
 int
 gnome_print_show_ucs4 (GnomePrintContext *pc, guint32 *buf, gint length)
 {
-	guchar *utf, *p;
-	gint i;
-
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
 	g_return_val_if_fail (buf != NULL, -1);
@@ -513,16 +521,21 @@ gnome_print_show_ucs4 (GnomePrintContext *pc, guint32 *buf, gint length)
 		warned = TRUE;
 	}
 
-	utf = p = alloca (length * 2);
+	if (PRINT_CLASS (pc)->show_sized) {
+		guchar *utf, *p;
+		gint i;
+		gint retval;
 
-	for (i = 0; i < length; i++) {
-		p += g_unichar_to_utf8 (GINT_FROM_BE (buf[i]), p);
+		utf = g_malloc (length * 2);
+		for (p = utf, i = 0; i < length; i++) {
+			p += g_unichar_to_utf8 (GINT_FROM_BE (buf[i]), p);
+		}
+		retval = (* PRINT_CLASS (pc)->show_sized) (pc, utf, p - utf);
+		g_free (utf);
+		return retval;
+	} else {
+		return GNOME_PRINT_OK;
 	}
-
-	if (PRINT_CLASS (pc)->show_sized)
-		return (* PRINT_CLASS (pc)->show_sized) (pc, utf, p - utf);
-
-	return GNOME_PRINT_OK;
 }
 
 int
