@@ -103,7 +103,7 @@ set_profile (GnomePrinterWidget *gpw, GnomePrinterProfile *pp)
 	} else {
 		const char *msg;
 		
-		if (strncmp (output, "file", 4) == 0)
+		if ((output != NULL) && (strncmp (output, "file", 4) == 0))
 			msg = output + 5;
 		else
 			msg = "output.ps";
