@@ -1,6 +1,9 @@
 #define __GNOME_RFONT_C__
 
 #include <math.h>
+#include <string.h>
+#include <ctype.h>
+#include <gdk/gdkx.h>
 #include <libgnomeprint/gnome-rfont.h>
 #include <libart_lgpl/art_misc.h>
 #include <libart_lgpl/art_affine.h>
@@ -25,8 +28,9 @@ struct _GnomeRFont {
 	GHashTable * graymaps;
 	GHashTable * dimensions;
 	GHashTable * displays;
+	/* Best fitting X font */
 	GdkFont * gdk_font;
-	gchar * x_font_name;
+	gchar * gdk_name;
 };
 
 struct _GnomeRFontClass {
@@ -79,6 +83,12 @@ static gboolean rfont_free_bpath (gpointer key, gpointer value, gpointer data);
 static guint rfont_hash (gconstpointer key);
 static gboolean rfont_equal (gconstpointer key1, gconstpointer key2);
 
+/* DisplayFont stuff */
+
+static gboolean gdf_find_gdk_font (GnomeRFont * rfont);
+
+#define GDF_XFONT(f) ((f->gdk_font != NULL) || (gdf_find_gdk_font ((GnomeRFont *) f)))
+
 static GtkObjectClass * parent_class;
 
 GtkType
@@ -132,7 +142,7 @@ gnome_rfont_destroy (GtkObject * object)
 
 	g_hash_table_remove (rfonts, rfont);
 
-	if (rfont->x_font_name) g_free (rfont->x_font_name);
+	if (rfont->gdk_name) g_free (rfont->gdk_name);
 	if (rfont->gdk_font) gdk_font_unref (rfont->gdk_font);
 
 	if (rfont->displays) {
@@ -874,474 +884,44 @@ dsp_get_glyph_bitmap (const GnomeRFontDisplay * dsp, gint glyph)
 }
 
 /*
- * These are original GnomeDisplayFont routines
- *
- * We merge these eventually with GnomeRFont
+ * GnomeDisplayFont stuff follows
  */
 
-GHashTable *scaled_display_fonts = NULL;
-GHashTable *gnome_font_family_to_x_hash = NULL;
-GHashTable *gnome_font_x_to_family_hash = NULL;
-GHashTable *gnome_font_weight_to_string_hash = NULL;
-GHashTable *gnome_font_string_to_weight_hash = NULL;
+typedef struct _GDFMap GDFMap;
 
-typedef struct
-{
-  char *print_name;
-  char *x_name;
-} gnome_font_family_to_x_map;
-
-gnome_font_family_to_x_map gnome_font_family_to_x_maps[] =
-{
-  {"Bitstream Charter", "charter"},
-  {"Century Schoolbook L", "new century schoolbook"},
-  {"Courier", "courier"},
-  {"Dingbats", "dingbats"},
-  {"Helvetica", "helvetica"},
-  {"ITC Avant Garde Gothic", "gothic"},
-  {"ITC Bookman", "bookman"},
-  {"ITC Zapf Chancery", "zapf chancery"},
-  {"ITC Zapf Dingbats", "zapf dingbats"},
-  {"New Century Schoolbook", "new century schoolbook"},
-  {"Nimbus Mono L", "nimbus"}, /* times roman? */
-  {"Nimbus Roman No9 L", "nimbus"}, /* times roman? */
-  {"Nimbus Sans L", "nimbus"}, /* times roman? */
-  {"Palatino", "palatino"},
-  {"Standard Symbols L", "symbol"},
-  {"Symbol", "symbol"},
-  {"Times", "times"},
-  {"URW Bookman L", "bookman"},
-  {"URW Chancery L", "chancery"},
-  {"URW Gothic L", "gothic"},
-  {"URW Palladio L", "palladio"},
-  {0, 0}
+struct _GDFMap {
+	gchar * name; /* Full name */
+	gchar * x_name[6]; /* ["*"], name1, name2, ..., NULL */
 };
 
-typedef struct
-{
-  GnomeFontWeight weight;
-  char *name;
-} gnome_font_weight_to_string_map;
+/* Default is Helvetica, so we add times for serifed font lookups */
 
-gnome_font_weight_to_string_map gnome_font_weight_to_string_maps [] =
-{
-  {GNOME_FONT_EXTRA_LIGHT, "extralight"},
-  {GNOME_FONT_THIN, "thin"},
-  {GNOME_FONT_LIGHT, "light"},
-  {GNOME_FONT_BOOK, "regular"},
-  {GNOME_FONT_MEDIUM, "medium"},
-  {GNOME_FONT_SEMI, "demibold"},
-  {GNOME_FONT_BOLD, "bold"},
-  {GNOME_FONT_HEAVY, "heavy"},
-  {GNOME_FONT_EXTRABOLD, "extrabold"},
-  {GNOME_FONT_BLACK, "black"},
-  {GNOME_FONT_EXTRABLACK, "extrablack"},
-  {(GnomeFontWeight)0, 0}
+GDFMap gp_2_x_map[] = {
+	{"Bitstream Charter", {"bitstream charter", "charter", "times", "times new roman", NULL}},
+	{"Century Schoolbook L", {"century schoolbook", "new century schoolbook", "times", "times new roman", NULL}},
+	{"Courier", {"courier", "nimbus mono", NULL}},
+	{"ITC Avant Garde Gothic", {"goth", NULL}},
+	{"ITC Bookman", {"bookman", "times", "times new roman", NULL}},
+	{"ITC Zapf Chancery", {"zapf chancery", "chancery", NULL}},
+	{"ITC Zapf Dingbats", {"zapf dingbats", "dingbats", NULL}},
+	{"New Century Schoolbook", {"new century schoolbook", "times", "times new roman", NULL}},
+	{"Nimbus Mono L", {"nimbus mono", "courier", NULL}},
+	{"Nimbus Roman No9 L", {"nimbus roman", "times", "times new roman", NULL}},
+	{"Nimbus Sans L", {"nimbus sans", "helvetica", NULL}},
+	{"Palatino", {"palatino", "times", "times new roman", NULL}},
+	{"Standard Symbols L", {"standard symbols", "symbol", NULL}},
+	{"Symbol", {"symbol", "standard symbols", NULL}},
+	{"Times", {"times", "times new roman", NULL}},
+	{"URW Bookman L", {"bookman", "times", "times new roman", NULL}},
+	{"URW Chancery L", {"chancery", "zapf chancery", NULL}},
+	{"URW Gothic L", {"goth", NULL}},
+	{"URW Palladio L", {"palladio", "times", "times new roman", NULL}},
+	{"Utopia", {"utopia", "times", "times new roman", NULL}},
+	{"Dingbats", {"dingbats", "zapf dingbats", NULL}},
+	{"Helvetica", {"helvetica", "nimbus sans", NULL}},
 };
 
-
-static void
-initialize_hashes (void)
-{
-  gnome_font_family_to_x_map *ftx = gnome_font_family_to_x_maps;
-  gnome_font_weight_to_string_map *wtx = gnome_font_weight_to_string_maps;
-
-  scaled_display_fonts = g_hash_table_new (g_str_hash, g_str_equal);
-  gnome_font_family_to_x_hash = g_hash_table_new (g_str_hash, g_str_equal);
-  gnome_font_x_to_family_hash = g_hash_table_new (g_str_hash, g_str_equal);
-  gnome_font_weight_to_string_hash =
-    g_hash_table_new (g_int_hash, g_int_equal);
-  gnome_font_string_to_weight_hash =
-    g_hash_table_new (g_str_hash, g_str_equal);
-
-  /* build 2 way hashes for x vs font family names */
-  while (ftx->print_name)
-    {
-#if 0
-      debugmsg ("inserting family-->x mapping: '%s' --> '%s'\n",
-		ftx->print_name, ftx->x_name);
-#endif
-      g_hash_table_insert (gnome_font_family_to_x_hash,
-			   ftx->print_name, ftx->x_name);
-      g_hash_table_insert (gnome_font_x_to_family_hash,
-			   ftx->x_name, ftx->print_name);
-      ftx ++;
-    }
-
-
-  /* build 2 way hashes for gnome font weight to x names */
-  while (wtx->name)
-    {
-#if 0
-      debugmsg ("inserting weight-->string mapping: '%d' --> '%s'\n",
-	      wtx->weight, wtx->name);
-#endif
-      g_hash_table_insert (gnome_font_weight_to_string_hash,
-			   & (wtx->weight), wtx->name);
-      g_hash_table_insert (gnome_font_string_to_weight_hash,
-			   wtx->name, & (wtx->weight));
-      wtx ++;
-    }
-}
-
-
-static const char *
-gnome_font_family_to_x_name (const char *family)
-{
-  const char *x_name;
-  if (! scaled_display_fonts) initialize_hashes ();
-
-  x_name = (const char *) g_hash_table_lookup (gnome_font_family_to_x_hash, family);
-  if (! x_name)
-    return "helvetica";
-
-  return x_name;
-}
-
-
-const char *
-gnome_font_weight_to_string (GnomeFontWeight gfw)
-{
-  const char *x_weight;
-  if (! scaled_display_fonts) initialize_hashes ();
-
-  x_weight =
-    (const char *) g_hash_table_lookup (gnome_font_weight_to_string_hash, &gfw);
-
-  if (! x_weight)
-    x_weight = "medium";
-
-  return x_weight;
-}
-
-
-GnomeFontWeight
-string_to_gnome_font_weight (const char *x_weight)
-{
-  GnomeFontWeight *gfw;
-
-  if (! scaled_display_fonts) initialize_hashes ();
-
-  gfw =
-    (GnomeFontWeight *)
-    g_hash_table_lookup (gnome_font_string_to_weight_hash, x_weight);
-
-  return (*gfw);
-}
-
-
-static char *
-setFontComponent (const char *font_name, unsigned int pos, const char *s)
-{
-  char *p;
-  const char *parts[ 14 ];
-  char *scratch;
-  char *new_name;
-  int i, len;
-
-  if (pos > 13)
-    {
-      g_warning ("replaceFontComponent -- pos out of ");
-      g_warning ("range: %d\n", pos);
-      return g_strdup (font_name);
-    }
-
-  if (font_name != NULL && *font_name != '\0')
-    scratch = g_strdup (font_name);
-  else
-    {
-      char *d = "-*-*-*-*-*-*-*-*-*-*-*-*-*-*";
-      scratch = g_strdup (d);
-    }
-
-  for (i=0, p=scratch+1; i<14 && (*p) != '\0'; i++)
-    {
-      parts[ i ] = p;
-      for (; *p != '-' && *p != '\0'; p ++)
-	;
-      *p = '\0';
-      p ++;
-    }
-
-  if (s == NULL)
-    parts[ pos ] = "*";
-  else
-    parts[ pos ] = s;
-
-  for (i=0, len=0; i<14; i++)
-    len += strlen (parts[ i ]);
-
-  new_name = (char *) g_malloc (len + 17);
-
-  g_snprintf(new_name, len + 17, "-%s-%s-%s-%s-%s-%s-%s-%s-%s-%s-%s-%s-%s-%s",
-	  parts[ 0 ], parts[ 1 ], parts[ 2 ], parts[ 3 ], 
-	  parts[ 4 ], parts[ 5 ], parts[ 6 ], parts[ 7 ], 
-	  parts[ 8 ], parts[ 9 ], parts[ 10 ], parts[ 11 ],
-	  parts[ 12 ], parts[ 13 ]);
-
-  g_free (scratch);
-  return new_name;
-}
-
-
-static char *
-setFontFamily (const char *font_name, const char *s)
-{
-  return setFontComponent (font_name, 1, s);
-}
-
-static char *
-setFontWeight (const char *font_name, const char *s)
-{
-  return setFontComponent (font_name, 2, s);
-}
-
-static char *
-setFontSlant (const char *font_name, const char *s)
-{
-  return setFontComponent (font_name, 3, s);
-}
-
-
-static char *
-setFontPixelSize (const char *font_name, int i)
-{
-  gchar temp[4 * sizeof (int)];
-  gchar *returnval;
-  g_snprintf (temp, 4 * sizeof (int), "%d", i );
-  returnval = setFontComponent (font_name, 6, temp);
-  return returnval;
-}
-
-
-static void
-setComponentReplace (char **io,
-		     char *(*func) (const char *font_name, const char *s),
-		     const char *part)
-{
-  char *new_name = (*func) (*io, part);
-  g_free (*io);
-  (*io) = new_name;
-}
-
-static GdkFont *
-find_best_x_weight (const char *x_font_name_ret, GnomeFontWeight start_weight)
-{
-  gchar *current_x_name = g_strdup( x_font_name_ret );
-  GdkFont *current_font = gdk_font_load (current_x_name);
-  GnomeFontWeight weighta, weightb;
-  const char *x_weight;
-
-  if (! scaled_display_fonts) initialize_hashes ();
-
-  weighta = weightb = start_weight;
-  if ( current_font )
-    {
-      g_free( current_x_name );
-      return current_font;
-    }
-  while( weighta >= GNOME_FONT_LIGHTEST || weightb <= GNOME_FONT_HEAVIEST )
-    {
-      weighta --;
-      weightb ++;
-
-      x_weight =
-	(const char *) g_hash_table_lookup (gnome_font_weight_to_string_hash, &weighta);
-
-      if ( x_weight )
-	{
-	  setComponentReplace (&current_x_name, setFontWeight, x_weight);
-	  current_font = gdk_font_load (current_x_name);
-	  if ( current_font )
-	    {
-	      g_free( current_x_name );
-	      return current_font;
-	    }
-	}
-
-      x_weight =
-	(const char *) g_hash_table_lookup (gnome_font_weight_to_string_hash, &weightb);
-
-      if ( x_weight )
-	{
-	  setComponentReplace (&current_x_name, setFontWeight, x_weight);
-	  current_font = gdk_font_load (current_x_name);
-	  if ( current_font )
-	    {
-	      g_free( current_x_name );
-	      return current_font;
-	    }
-	}
-  
-    }
-  
-  g_free( current_x_name );
-  return NULL;
-}
-
-/* here is a horrible little routine that tries to find an X font
-   that is about the same size as a printer font */
-static GdkFont *find_best_x_font (const char *unsized_name,
-				  GnomeFont *gnome_font,
-				  char **x_font_name_ret)
-{
-  /* sample string used during the search for a good size */
-  /*char *test_string = "abcdefghijklmnopqrstuvwxyz";*/
-  /*char *test_string = "snider";*/
-  /* const char *test_string = "oooowwww"; */
-#if 0
-  int length = strlen (test_string);
-
-  double perfect_width; /* the printer-font width of the sample string */
-  double max_width; /* we must end up below this size */
-
-  int direction; /* either +1 or -1 -- are we looking for a bigger font
-		    or a smaller one? */
-
-  /* information about the best match found so far */
-  int current_width; /* width of rendered sample string */
-  int current_diff; /* difference between current_width and perfect_width */
-#endif
-  int current_size; /* point size */
-  char *current_x_name; /* the x-style name of the current font */
-  GdkFont *current_font; /* the current font */
-
-#if 0
-  g_return_val_if_fail (unsized_name, NULL);
-  g_return_val_if_fail (gnome_font, NULL);
-  g_return_val_if_fail (x_font_name_ret, NULL);
-
-  perfect_width = gnome_font_get_width_string (gnome_font, test_string);
-  /*space_width = gnome_font_get_width_string (gnome_font, " ");*/
-  /*max_width = perfect_width + space_width / 2;*/
-  max_width = perfect_width /*- 1*/;
-  debugmsg( "perfect_width = %f\n", perfect_width );
-#endif
-  current_size = gnome_font_get_size (gnome_font);
-  current_x_name = setFontPixelSize (unsized_name, current_size);
-  current_font = find_best_x_weight (current_x_name, gnome_font->private->fontmap_entry->private->weight_code);
-
-  /* start by getting some x font loaded */
-  if (current_font == NULL)
-    {
-#if 0
-      debugmsg ("couldn't load first font '%s'\n", current_x_name);
-#endif
-      /* is it italic? */
-      if (gnome_font->private->fontmap_entry->private->italic)
-	{
-	  /* try oblique */
-#if 0
-	  debugmsg ("trying oblique...\n");
-#endif
-	  setComponentReplace (&current_x_name, setFontSlant, "o");
-	  current_font = find_best_x_weight (current_x_name, gnome_font->private->fontmap_entry->private->weight_code);
-
-	  if (current_font == NULL)
-	    {
-	      /* failed.  put italic back */
-#if 0
-	      debugmsg ("oblique failed: '%s'\n", current_x_name);
-#endif
-	      setComponentReplace (&current_x_name, setFontSlant, "i");
-	    }
-	}
-
-      if (current_font == NULL)
-	{
-	  /* try helvetica */
-#if 0
-	  debugmsg ("trying hevletica...\n");
-#endif
-	  setComponentReplace (&current_x_name, setFontFamily, "helvetica");
-	  current_font = find_best_x_weight (current_x_name, gnome_font->private->fontmap_entry->private->weight_code);
-
-	  if (current_font == NULL)
-	    {
-#if 0
-	      debugmsg ("helvetica failed. giving up...\n");
-#endif
-	      return NULL;
-	    }
-	}
-    }
-#if 0
-  debugmsg ("initial name loaded: '%s'\n", current_x_name);
-#endif
-#if 0
-  current_width = gdk_text_width (current_font, test_string, length);
-  current_diff = (int) (perfect_width - current_width);
-
-  debugmsg ("size=%d, starting diff=%d\n", current_size, current_diff);
-
-  /* are we looking for a bigger x font or a smaller one? */
-  /*
-  if (current_diff > 0) direction = 1;
-  else direction = -1;
-  */
-  if (current_diff > 0)
-    {
-      /*direction = (int) gnome_font->size / 6.0;*/
-      direction = 10;
-      /*if (direction < 1) direction = 1;*/
-      debugmsg ("size=%f, direction=%d\n", gnome_font->size, direction);
-    }
-  else
-    {
-      /*direction = (int) gnome_font->size / -6.0;*/
-      direction = -10;
-      /*if (direction > -1) direction = -1;*/
-      debugmsg ("size=%f, direction=%d\n", gnome_font->size, direction);
-    }
-
-
-  while (current_diff != 0)
-    {
-      int next_size = current_size + direction;
-      char *next_x_name = setFontPixelSize (current_x_name, next_size);
-      GdkFont *next_font = find_best_x_weight (next_x_name, gnome_font->fontmap_entry->weight_code);
-      int next_width;
-      int next_diff;
-
-      if (next_font == NULL)
-	{
-	  debugmsg ("couldn't load '%s'\n", next_x_name);
-	  continue; /* ??? */
-	}
-      next_width = gdk_text_width (next_font, test_string, length);
-      next_diff = (int) (perfect_width - next_width);
-
-      /* if we've gotten bigger than max_width, stop */
-      if (next_width >= max_width && direction > 0)
-	{
-	  debugmsg (" bailing because next_width[%d] >= max_width[%d]\n",
-		  next_width, (int) max_width);
-	  break;
-	}
-
-      /* if the previous font was closer than this one, stop */
-      if (iabs (next_diff) > iabs (current_diff))
-	{
-	  debugmsg (" bailing because next_diff[%d] > current_diff[%d]\n",
-		  next_diff, current_diff);
-	  break;
-	}
-
-      g_free (current_x_name);
-      gdk_font_unref (current_font);
-
-      current_size = next_size;
-      current_x_name = next_x_name;
-      current_font = next_font;
-      current_diff = next_diff;
-
-      debugmsg (" new size=%d, new diff=%d\n", current_size, current_diff);
-    }
-#endif
-  (*x_font_name_ret) = current_x_name;
-  return current_font;
-}
-
+#define gp_2_x_map_size (sizeof (gp_2_x_map) / sizeof (gp_2_x_map[0]))
 
 
 /* construct an x font name and find the best point size */
@@ -1352,51 +932,25 @@ static GnomeDisplayFont *create_display_font (const char *family,
 					      double points,
 					      double scale)
 {
-	GnomeFont *gnome_font;
-#if 0
-  GnomeFont *closest;
-#endif
-#if 0
-  GnomeFontUnsized *gfus;
-  GnomeDisplayFont *wpdf;
-#else
-#if 0
-  GnomeFontFace * face;
-#endif
-  GnomeRFont * rfont;
-  gdouble transform[6];
-#endif
+	GnomeFont * gnomefont;
+	GnomeRFont * rfont;
+	gdouble transform[6];
 
-  g_return_val_if_fail (family, NULL);
-
-  if (! scaled_display_fonts) initialize_hashes ();
+	g_return_val_if_fail (family, NULL);
 
 #if 0
-  if ((closest = gnome_font_new_closest (family, weight, italic, 0)) == NULL)
-	  return NULL;
+	if (! scaled_display_fonts) initialize_hashes ();
 #endif
 
-  gnome_font = gnome_font_new_closest (family, weight, italic, ceil (points * scale));
+	gnomefont = gnome_font_new_closest (family, weight, italic, ceil (points * scale));
+	g_return_val_if_fail (gnomefont != NULL, NULL);
 
-  g_return_val_if_fail (gnome_font != NULL, NULL);
+	art_affine_scale (transform, 1.0, -1.0);
+	rfont = gnome_font_get_rfont (gnomefont, transform);
 
-#if 0	/* This is crashing and we don't use */
-          /*face in the rest of the function. (Chema)*/
-  face = closest->private->fontmap_entry;
-#endif
+	gnome_font_unref (gnomefont);
 
-  art_affine_scale (transform, scale, -scale);
-  rfont = gnome_font_get_rfont (gnome_font, transform);
-
-	  gnome_font_unref (gnome_font);
-#if 0
-  wpdf = (GnomeDisplayFont *) g_malloc (sizeof (GnomeDisplayFont));
-  wpdf->unsized_font = gfus;
-  wpdf->gnome_font = gnome_font;
-  wpdf->scale = scale;
-#endif
-
-  return rfont;
+	return rfont;
 }
 
 /* cache for create_display_font */
@@ -1407,33 +961,9 @@ GnomeDisplayFont *gnome_get_display_font (const char *family,
 					  double points,
 					  double scale)
 {
-  char key[ 1024 ];
   GnomeDisplayFont *wpdf;
 
-  g_snprintf (key, sizeof (key), "%s.%s.%s.%d",
-	      family,
-	      gnome_font_weight_to_string (weight),
-	      italic?"t":"f",
-	      (int) ceil (points * scale));
-
-  if (! scaled_display_fonts) initialize_hashes ();
-
-  /*
-  debugmsg ("lookup_display_font -- hash:%p key='%s'\n",
-	  scaled_display_fonts,
-	  key);
-  */
-  wpdf =
-    (GnomeDisplayFont *) g_hash_table_lookup (scaled_display_fonts, key);
-
-  if (wpdf == NULL)
-    {
       wpdf = create_display_font (family, weight, italic, points, scale);
-      if (wpdf)
-	g_hash_table_insert (scaled_display_fonts, g_strdup (key), wpdf);
-      else
-	/* We should maintain a negative cache.  */;
-    }
 
   return wpdf;
 }
@@ -1443,46 +973,25 @@ gnome_display_font_height (GnomeDisplayFont * gdf)
 {
 	g_return_val_if_fail (gdf != NULL, 0);
 	g_return_val_if_fail (gdf->gdk_font != NULL, 0);
+	g_return_val_if_fail (GDF_XFONT (gdf), 0);
 
-	return MAX (gdf->gdk_font->ascent + gdf->gdk_font->descent,
-		    gdf->font->private->size);
+	/* We return GnomeFont height here */
+	return MAX (gnome_font_get_size (gdf->font) * gdf->transform[0], gdf->gdk_font->ascent + gdf->gdk_font->descent);
 }
 
-GnomeDisplayFont *gnome_font_get_display_font( GnomeFont *font )
+GnomeDisplayFont * gnome_font_get_display_font (GnomeFont * font)
 {
   return gnome_get_display_font( font->private->fontmap_entry->private->familyname,
 				 font->private->fontmap_entry->private->weight_code,
 				 font->private->fontmap_entry->private->italic,
 				 font->private->size,
-				 font->private->scale );
+				 1.0);
 }
 
 /*
  * Here are GnomeDisplayFont methods
  */
 
-static void
-gdf_find_x_fonts (GnomeDisplayFont * gdf)
-{
-	const gchar * x_family;
-	gchar * xfn;
-
-	x_family = gnome_font_family_to_x_name (gdf->face->private->familyname);
-	xfn = g_strdup ("-*-helvetica-*-r-*-*-12-*-*-*-*-*-*-*");
-
-	setComponentReplace (&xfn, setFontFamily, x_family);
-	setComponentReplace (&xfn, setFontWeight,
-		       gnome_font_weight_to_string (gdf->face->private->weight_code));
-
-	if (gdf->face->private->italic)
-		setComponentReplace (&xfn, setFontSlant, "i");
-
-
-	gdf->gdk_font = find_best_x_font (xfn, gdf->font, &(gdf->x_font_name));
-
-	g_free (xfn);
-}
-
 const GnomeFontFace *
 gnome_display_font_get_face (const GnomeDisplayFont * gdf)
 {
@@ -1507,7 +1016,7 @@ gnome_display_font_get_scale (const GnomeDisplayFont * gdf)
 	g_return_val_if_fail (gdf != NULL, 0.0);
 	g_return_val_if_fail (GNOME_IS_RFONT (gdf), 0.0);
 
-	return art_affine_expansion (gdf->transform);
+	return gdf->transform[0];
 }
 
 const gchar *
@@ -1515,12 +1024,9 @@ gnome_display_font_get_x_font_name (const GnomeDisplayFont * gdf)
 {
 	g_return_val_if_fail (gdf != NULL, NULL);
 	g_return_val_if_fail (GNOME_IS_RFONT (gdf), NULL);
+	g_return_val_if_fail (GDF_XFONT (gdf), NULL);
 
-	if (!gdf->x_font_name) {
-		gdf_find_x_fonts ((GnomeDisplayFont *) gdf);
-	}
-
-	return gdf->x_font_name;
+	return gdf->gdk_name;
 }
 
 GdkFont *
@@ -1528,10 +1034,7 @@ gnome_display_font_get_gdk_font (const GnomeDisplayFont * gdf)
 {
 	g_return_val_if_fail (gdf != NULL, NULL);
 	g_return_val_if_fail (GNOME_IS_RFONT (gdf), NULL);
-
-	if (!gdf->x_font_name) {
-		gdf_find_x_fonts ((GnomeDisplayFont *) gdf);
-	}
+	g_return_val_if_fail (GDF_XFONT (gdf), NULL);
 
 	return gdf->gdk_font;
 }
@@ -1548,9 +1051,342 @@ gnome_display_font_unref (GnomeDisplayFont * gdf)
 	gtk_object_unref (GTK_OBJECT (gdf));
 }
 
+static void gdf_find_measured_gdk_font (const gchar * name, GnomeFontWeight weight, gboolean italic, gdouble size,
+					GdkFont ** gdkfont, gchar ** gdkname, gint * best);
+static void gdf_measure_string_name (const gchar * xname,
+				     const gchar * familyname, GnomeFontWeight weight, gboolean italic, gdouble size,
+				     gchar ** bestname, gint * best);
+static gint gdf_measure_x_name (const gchar * xname, GnomeFontWeight weight, gboolean italic, gdouble size);
+static void split_name (gchar * c[], gchar * name);
 
+#ifdef VERBOSE
+/*
+ * Return newly allocated full name
+ */
 
+static gchar *
+get_font_name (const GdkFont * font)
+{
+	Atom font_atom, atom;
+	Bool status;
 
+#ifdef VERBOSE
+	g_print ("Extracting X font info\n");
+#endif
 
+	font_atom = gdk_atom_intern ("FONT", FALSE);
+
+	if (font->type == GDK_FONT_FONTSET) {
+		XFontStruct **font_structs;
+		gint num_fonts;
+		gchar **font_names;
+
+		num_fonts = XFontsOfFontSet (GDK_FONT_XFONT (font), &font_structs, &font_names);
+		status = XGetFontProperty (font_structs[0], font_atom, &atom);
+	} else {
+		status = XGetFontProperty (GDK_FONT_XFONT (font), font_atom, &atom);
+	}
+
+	if (status) {
+		return gdk_atom_name (atom);
+	}
+
+	return NULL;
+}
+#endif
+
+static gboolean
+gdf_find_gdk_font (GnomeRFont * rfont)
+{
+	static GHashTable * gp2x = NULL;
+	const gchar * familyname;
+	GnomeFontWeight weight;
+	gboolean italic;
+	gdouble size;
+	gint best;
+	GdkFont * gdkfont;
+	gchar * gdkname = NULL, * xname;
+
+	if (rfont->gdk_font) return TRUE;
+
+	familyname = gnome_font_face_get_family_name (rfont->face);
+	weight = gnome_font_face_get_weight_code (rfont->face);
+	italic = gnome_font_face_is_italic (rfont->face);
+	size = gnome_font_get_size (rfont->font) * rfont->transform[0];
+
+	if (!gp2x) {
+		gint i;
+		gp2x = g_hash_table_new (g_str_hash, g_str_equal);
+		for (i = 0; i < gp_2_x_map_size; i++) {
+			g_hash_table_insert (gp2x, gp_2_x_map[i].name, gp_2_x_map[i].x_name);
+		}
+	}
+
+	best = G_MAXINT;
+	gdkfont = NULL;
+
+	/* First try with original font name */
+
+	xname = g_strdup (familyname);
+	g_strdown (xname);
+	gdf_find_measured_gdk_font (xname, weight, italic, size, &gdkfont, &gdkname, &best);
+	g_free (xname);
+
+	if (best > 0) {
+		gchar ** xn;
+
+		/* Try with known alternative names */
+
+		xn = g_hash_table_lookup (gp2x, familyname);
+		if (xn) {
+			while (*xn) {
+				gdf_find_measured_gdk_font (*xn, weight, italic, size, &gdkfont, &gdkname, &best);
+				if (best == 0) break;
+				xn++;
+			}
+		}
+	}
+
+	if (!gdkfont) {
+		/* Try Helvetica */
+
+		gdf_find_measured_gdk_font ("helvetica", weight, italic, size, &gdkfont, &gdkname, &best);
+	}
+
+	if (!gdkfont) {
+		/* No useful font found, so load 'fixed' */
+		
+		gdkfont = gdk_font_load ("fixed");
+		gdkname = g_strdup ("fixed");
+	}
+
+	rfont->gdk_font = gdkfont;
+	rfont->gdk_name = gdkname;
+
+#ifdef VERBOSE
+	g_print ("Asked: %s %d %d %f\n", familyname, weight, italic, size);
+	g_print ("Tried: %s\n", gdkname);
+	xname = get_font_name (gdkfont);
+	g_print ("Got: %s\n", xname);
+	if (xname) g_free (xname);
+#endif
+
+	return TRUE;
+}
+
+static void
+gdf_find_measured_gdk_font (const gchar * name, GnomeFontWeight weight, gboolean italic, gdouble size,
+			    GdkFont ** gdkfont, gchar ** gdkname, gint * best)
+{
+	gchar xname[1024];
+	gint pxsize;
+	gchar * bestname;
+
+	g_return_if_fail (strlen (name) < 512);
+
+	bestname = NULL;
+	pxsize = 0;
+
+	/* Try 0-sized font */
+
+	g_snprintf (xname, 1024, "-%s-%s-%s-%s-%s-%s-%d-%s-%s-%s-%s-%s-%s",
+		    "*", /* Foundry */
+		    name, /* Family */
+		    "*", /* Weight */
+		    "*", /* Slant */
+		    "normal", /* Set width */
+		    "*", /* Add Style */
+		    pxsize, /* Pixel Size */
+		    "*", /* Point size */
+		    "*", "*", /* Resolution X, Y */
+		    "*", "*", "*"); /* Spacing, Avg Width, Charset */
+
+	gdf_measure_string_name (xname, name, weight, italic, size, &bestname, best);
+
+	if (!bestname) {
+		/* Try exact size */
+		pxsize = (gint) floor (size + 0.5);
+		g_snprintf (xname, 1024, "-%s-%s-%s-%s-%s-%s-%d-%s-%s-%s-%s-%s-%s",
+			    "*", name, "*", "*", "normal", "*", pxsize, "*", "*", "*", "*", "*", "*");
+		gdf_measure_string_name (xname, name, weight, italic, size, &bestname, best);
+	}
+
+	if (!bestname) {
+		/* We didn't find any name - try walking up'n'down */
+		gint dev, i;
+		dev = pxsize / 3;
+		for (i = 1; i <= dev; i++) {
+			g_snprintf (xname, 1024, "-%s-%s-%s-%s-%s-%s-%d-%s-%s-%s-%s-%s-%s",
+				    "*", name, "*", "*", "normal", "*", (pxsize + i), "*", "*", "*", "*", "*", "*");
+			gdf_measure_string_name (xname, name, weight, italic, size, &bestname, best);
+			if (*best == 0) break;
+			g_snprintf (xname, 1024, "-%s-%s-%s-%s-%s-%s-%d-%s-%s-%s-%s-%s-%s",
+				    "*", name, "*", "*", "normal", "*", (pxsize - i), "*", "*", "*", "*", "*", "*");
+			gdf_measure_string_name (xname, name, weight, italic, size, &bestname, best);
+			if (*best == 0) break;
+		}
+	}
+
+	if (bestname) {
+		GdkFont * f;
+		gchar * c[14];
+
+		split_name (c, bestname);
+
+		/* If font is scalable, use exact size */
+		if (pxsize == 0) pxsize = (gint) floor (size + 0.5);
+
+		/* Clear unimportant fields */
+		g_snprintf (xname, 1024, "-*-%s-%s-%s-%s-*-%d-*-*-*-*-*-%s",
+			    c[2], c[3], c[4], c[5], pxsize, "*" /* c[13] */);
+#ifdef VERBOSE
+		g_print ("Trying: %s\n", xname);
+#endif
+		f = gdk_font_load (xname);
+		if (!f) {
+			g_snprintf (xname, 1024, "-*-%s-%s-%s-%s-*-%d-*-*-*-*-*-%s",
+				    c[2], c[3], c[4], c[5], pxsize, c[13]);
+#ifdef VERBOSE
+			g_print ("Trying: %s\n", xname);
+#endif
+			f = gdk_font_load (xname);
+		}
+
+		if (f) {
+#ifdef VERBOSE
+			g_print ("Got it\n");
+#endif
+			if (*gdkname) g_free (*gdkname);
+			*gdkname = g_strdup (xname);
+			if (*gdkfont) gdk_font_unref (*gdkfont);
+			*gdkfont = f;
+		}
+
+		g_free (bestname);
+	}
+}
+
+static void
+gdf_measure_string_name (const gchar * xname,
+			 const gchar * familyname, GnomeFontWeight weight, gboolean italic, gdouble size,
+			 gchar ** bestname, gint * best)
+{
+	gchar ** namelist;
+	gint numfonts;
+
+	namelist = XListFonts (GDK_DISPLAY (), xname, 256, &numfonts);
+
+	if (namelist) {
+		gchar * nptr;
+		gint i;
+
+		nptr = NULL;
+
+		for (i = 0; i < numfonts; i++) {
+			gint dist;
+
+			dist = gdf_measure_x_name (namelist[i], weight, italic, size);
+
+			if (dist < *best) {
+				*best = dist;
+				nptr = namelist[i];
+				if (*best == 0) break;
+			}
+		}
+
+		if (nptr) {
+			if (*bestname) g_free (*bestname);
+			*bestname = g_strdup (nptr);
+		}
+
+		XFreeFontNames (namelist);
+	}
+}
+
+static gint
+gdf_measure_x_name (const gchar * xname, GnomeFontWeight weight, gboolean italic, gdouble size)
+{
+	static GHashTable * xw2gw = NULL;
+	gchar * c[14];
+	gchar * p;
+	gdouble actualsize, actualweight;
+	gdouble dist_size, dist_weight, dist_italic;
+
+	p = g_strdup (xname);
+	split_name (c, p);
+
+	/* <size> = (((actualsize - reqsize) / reqsize) / 0.05) ^ 2 */
+
+	actualsize = atof (c[7]);
+
+	if (actualsize == 0.0) {
+		/* 0 means fonts is scalable */
+		dist_size = 0.0;
+	} else if (floor (size + 0.5) == actualsize) {
+		/* We differ no more than 0.5 pixel */
+		dist_size = 0.0;
+	} else {
+		dist_size = (((actualsize - size) / size) / 0.05);
+		dist_size = dist_size * dist_size;
+	}
+
+	if (!xw2gw) {
+		xw2gw = g_hash_table_new (g_str_hash, g_str_equal);
+		g_hash_table_insert (xw2gw, "extralight", GINT_TO_POINTER (GNOME_FONT_EXTRA_LIGHT));
+		g_hash_table_insert (xw2gw, "thin", GINT_TO_POINTER (GNOME_FONT_THIN));
+		g_hash_table_insert (xw2gw, "light", GINT_TO_POINTER (GNOME_FONT_LIGHT));
+		g_hash_table_insert (xw2gw, "regular", GINT_TO_POINTER (GNOME_FONT_BOOK));
+		g_hash_table_insert (xw2gw, "medium", GINT_TO_POINTER (GNOME_FONT_MEDIUM));
+		g_hash_table_insert (xw2gw, "demibold", GINT_TO_POINTER (GNOME_FONT_SEMI));
+		g_hash_table_insert (xw2gw, "bold", GINT_TO_POINTER (GNOME_FONT_BOLD));
+		g_hash_table_insert (xw2gw, "heavy", GINT_TO_POINTER (GNOME_FONT_HEAVY));
+		g_hash_table_insert (xw2gw, "extrabold", GINT_TO_POINTER (GNOME_FONT_EXTRABOLD));
+		g_hash_table_insert (xw2gw, "black", GINT_TO_POINTER (GNOME_FONT_BLACK));
+		g_hash_table_insert (xw2gw, "extrablack", GINT_TO_POINTER (GNOME_FONT_EXTRABLACK));
+	}
+
+	/* <weight> = actualweight - reqweight */
+
+	g_strdown (c[3]);
+	actualweight = (gdouble) GPOINTER_TO_INT (g_hash_table_lookup (xw2gw, c[3]));
+	dist_weight = actualweight - (gdouble) weight;
+
+	/* <italic> = 0 (i-i, r-r), 5 (i-r, r-i) */
+
+	if (italic) {
+		if ((tolower (*c[4]) == 'i') || (tolower (*c[4]) == 'o')) dist_italic = 0.0;
+		else dist_italic = 5.0;
+	} else {
+		if (tolower (*c[4]) == 'r') dist_italic = 0.0;
+		else dist_italic = 5.0;
+	}
+
+	g_free (p);
+
+	return (gint) floor (sqrt (dist_size * dist_size + dist_weight * dist_weight + dist_italic * dist_italic) + 0.5);
+}
+
+/*
+ * Splits full X font name into pieces, overwriting hyphens
+ */
+
+static void
+split_name (gchar * c[], gchar * name)
+{
+	gchar *p;
+	gint i;
+
+	p = name;
+
+	for (i = 0; i < 13; i++) {
+		c[i] = p;
+		/* Skip text */
+		while (*p && (*p != '-')) p++;
+		/* Replace hyphen with '\0' */
+		if (*p) *p++ = '\0';
+	}
+
+	c[i] = p;
+}
 
 
