@@ -290,8 +290,19 @@ gp_gc_set_linewidth (GPGC * gc, gdouble width)
 	ctx = (GPCtx *) gc->ctx->data;
 
 	/* Fixme: Currently we calculate average of pen size in both directions */
-
+#if 0
 	ctx->linewidth = (fabs (width * ctx->ctm[0]) + fabs (width * ctx->ctm[3])) / 2;
+#else
+	/* If we are in landscape mode the CTM is : 0 1 -1 0 0 0,
+	   so we need to take into account 0 thru 3. This is still not right.
+	   FIXME.
+	   see : ( run in "gs" ) http://www.gnome.org/~chema/line_width_problem
+	*/
+	ctx->linewidth = (fabs (width * ctx->ctm[0]) +
+				   fabs (width * ctx->ctm[1]) +
+				   fabs (width * ctx->ctm[2]) +
+				   fabs (width * ctx->ctm[3])) / 2;
+#endif
 
 	return 0;
 }
