@@ -3101,15 +3101,16 @@ gt1_load_font (const char *filename, GHashTable *privencoding)
 
 	  if (nche->name) {
 		  unival = gp_unicode_from_ps (nche->name);
+		  if (unival < 1) unival = gp_unicode_from_dingbats (nche->name);
 		  if (unival < 1) {
 			  if (privencoding) {
 				  unival = GPOINTER_TO_INT (g_hash_table_lookup (privencoding, nche->name));
 			  }
 		  }
 		  if (unival > 0) {
-			  if (gp_multi_from_ps (nche->name)) {
-				  const GSList * l;
-				  l = gp_multi_from_ps (nche->name);
+			  const GSList * l;
+			  l = gp_multi_from_ps (nche->name);
+			  if (l) {
 				  while (l) {
 					  g_hash_table_insert (encoding, l->data, GINT_TO_POINTER (nche->Gt1NameId));
 					  l = l->next;
