@@ -39,6 +39,8 @@ do_image (GnomePrintContext * pc, gdouble size, gboolean alpha)
 	guchar * image;
 	gint x, y;
 	guchar * p;
+	gint save_level = 300;
+	gint i;
 
 	image = g_new (guchar, 256 * 256 * 4);
 	for (y = 0; y < 256; y++) {
@@ -51,12 +53,16 @@ do_image (GnomePrintContext * pc, gdouble size, gboolean alpha)
 		}
 	}
 
-	gnome_print_gsave (pc);
+	for (i=0;i<save_level;i++)
+		gnome_print_gsave (pc);
+		
 	gnome_print_translate (pc, 0.0, size);
 	gnome_print_scale (pc, size, -size);
 	gnome_print_rgbaimage (pc, image, 256, 256, 4 * 256);
-	gnome_print_grestore (pc);
 
+	for (i=0;i<save_level;i++)
+		gnome_print_grestore (pc);
+	
 	g_free (image);
 }
 
@@ -102,6 +108,7 @@ do_rosette (GnomePrintContext * pc, GnomeFont * font)
 			       0xce, 0xb9, 0xce, 0xac,
 			       0x20, 0xcf, 0x83, 0xce,
 			       0xb1, 0xcf, 0x82, 0x00};
+	GnomeDisplayFont * df;
 	GnomeFont * tf, * cf;
 	GnomeGlyphList * gl;
 	gdouble angle;
@@ -109,7 +116,9 @@ do_rosette (GnomePrintContext * pc, GnomeFont * font)
 	gchar u[256];
 	gint len;
 
-	tf = gnome_font_new ("Times-Bold", 18.0);
+	tf = gnome_font_new ("Times-Bold", 19.0);
+	g_assert (df = gnome_get_display_font ("ITC Zapf Chancery", GNOME_FONT_BOOK, FALSE, 19.0, 1.0));
+	g_assert (gnome_display_font_get_gdk_font (df));
 	cf = gnome_font_new ("Symbol", 10.0);
 
 	gl = gnome_glyphlist_from_text_dumb (font, 0xff0000ff, 1.0, 0.0, "");
@@ -203,7 +212,7 @@ do_print (GnomePrintContext * pc, gdouble scale)
 	gnome_print_scale (pc, 3.0, 3.0);
 	gnome_print_newpath (pc);
 	gnome_print_moveto (pc, 20.0, 20.0);
-	gnome_print_show (pc, "Hello");
+/*	gnome_print_show (pc, "Hello");*/
 	gnome_print_grestore (pc);
 
 	gnome_print_newpath (pc);
